@@ -47,11 +47,15 @@
 #define	SIGEVTCHK	0x1010
 #define SIGBITCHK	0x1111
 
+
 #ifdef CCXX_NAMESPACES
 namespace ost {
 using namespace std;
 #endif
 
+const int HANGUP_TONE_ID=0x0200U;
+int hangup_detect=0;
+
 DialogicConfig::DialogicConfig() :
 Keydata("/bayonne/dialogic")
 {
@@ -69,8 +73,19 @@ Keydata("/bayonne/dialogic")
 	{"span", "e1"},
 	{"spans", "100"},
 	{"pickup", "800"},
+	{"hangup_detect", "off"},
+	{"hangup_freq", "425"},
+	{"hangup_freq_dev", "50"},
+	{"hangup_on", "30"},
+	{"hangup_on_dev", "15"},
+	{"hangup_off", "30"},
+	{"hangup_off_dev", "15"},
+	{"hangup_rep", "2"},
 	{NULL, NULL}};
 
+	if(isFHS())
+		load("/drivers/dialogic");
+
 	load(defkeys);
 }
 
@@ -92,19 +107,18 @@ Driver(), Thread(keythreads.priService() - 1)
 		{"wait", (Method)&DialogicTrunk::scrWait, &ScriptCommand::chkHasArgs},
 		{NULL, NULL, NULL}};
 
-	DialogicTrunk *trk;
 	char d4xname[32];
 	char dtiname[32];
 	int spans = 0;
 	int isdev = -1;
 	int dticards, voxcards, pricards;
-	int dticard = 1, voxcard = 1, digital = 0, pricard = 0, isdncard = 0;
-	int card = 0, dti = 0, is = 0;
-	int voxcount, vox, dticount;
+	int dticard = 1, voxcard = 1, pricard = 0, isdncard = 0;
+	int dti = 0;
+	int voxcount, vox, dticount = 0;
 	int id = 0;
-	int ts, tslots, chdev, tsdev;
+	int chdev, tsdev;
 	int maxslots = getTimeslots();
-	interface_t iface;
+	interface_t iface = NT_E1;
 	char *span;
 	CT_DEVINFO ctinfo;
 
@@ -200,6 +214,20 @@ Driver(), Thread(keythreads.priService() - 1)
 			//dx_chgrepcnt(TID_BUSY2, 3);
 			if(dx_initcallp(chdev))
 				slog(Slog::levelDebug) << "initcallp " << ATDV_ERRMSGP(chdev) << endl;
+			if(0==strcmp(getLast("hangup_detect"),"on"))
+			{
+				hangup_detect=1;
+				if(dx_bldstcad(HANGUP_TONE_ID, atoi(getLast("hangup_freq")),
+				  atoi(getLast("hangup_freq_dev")), atoi(getLast("hangup_on")),
+				  atoi(getLast("hangup_on_dev")), atoi(getLast("hangup_off")),
+				  atoi(getLast("hangup_off_dev")), atoi(getLast("hangup_rep"))))
+				    slog(Slog::levelDebug) << ATDV_ERRMSGP(chdev) << endl;
+				else
+				{
+				    if(dx_addtone(chdev, 0, 0))
+					slog(Slog::levelDebug) << ATDV_ERRMSGP(chdev) << endl;
+                        	}
+			}
 
 			if(!dti && vox == 1)
 			{
@@ -496,7 +524,6 @@ void DialogicDriver::run(void)
 	DV_DIGIT *digbuf;
 	CRN tmpcrn;
 	IE_BLK ie_blk;
-	char *buffer;
 	int statebuf, i, param;
 
 	unsigned short sig;
@@ -504,21 +531,13 @@ void DialogicDriver::run(void)
 	int evtype;
 	int digcnt, diglen;
 	unsigned long tmask;
-	char *d;
-	long ldata;
-	unsigned code;
 	unsigned delay = dialogicivr.getDelay();
-	void *datap;
+	void *datap = NULL;
 	unsigned char *alarmp;
-	int	len;
 	interface_t	iface;
 
 
-#ifdef	COMMON_THREAD_SLEEP
 	Thread::sleep(2000);
-#else
-	ccxx_sleep(2000);
-#endif
 	running = true;
 	slog(Slog::levelDebug) << "dialogic: service thread started" << endl;
 
@@ -529,11 +548,7 @@ void DialogicDriver::run(void)
 		if(sr_waitevt(EVT_TIMER) == -1)
 		{
 			RELEASE
-#ifdef	COMMON_THREAD_SLEEP
 			Thread::sleep(delay);
-#else
-			ccxx_sleep(delay);
-#endif
 			continue;
 		}
 		evtype = sr_getevttype();
@@ -703,7 +718,7 @@ void DialogicDriver::run(void)
 			switch(ATDX_CPTERM(trunk->chdev))
 			{
 			case CR_BUSY:
-				switch(ATDX_CRTNID(trunk->chdev))	
+				switch(ATDX_CRTNID(trunk->chdev))
 				{
 				case TID_BUSY1:
 					event.parm.tone.name = "busy";
@@ -896,6 +911,13 @@ void DialogicDriver::run(void)
 				event.parm.dtmf.digit = trunk->getDigit(cst->cst_data);
 				trunk->putEvent(&event);
 				break;
+			case DE_TONEON:
+				if(hangup_detect == 1 && cst->cst_data == HANGUP_TONE_ID)
+				{
+					event.id = TRUNK_STOP_DISCONNECT;
+					trunk->putEvent(&event);
+				}
+				break;
 			}
 			break;
 		case SR_TMOUTEVT:
@@ -1067,7 +1089,7 @@ void DialogicDriver::run(void)
 			slog(Slog::levelError) << "UNHANDLED EVENT: " << evtype << endl;
 		}
 		CONTINUE
-	} 
+	}
 }
 
 DialogicISDN *DialogicDriver::getISDN(int fd)
