@@ -57,11 +57,10 @@ using namespace std;
 char DialogicTrunk::status[600];
 
 DialogicTrunk::DialogicTrunk(int id, devices_t *devif) :
-Trunk(id, 0, devif->span), TimerPort(), Thread(0), URLAudio(), AudioService()
+TimerPort(), Trunk(id, 0, devif->span), Thread(0), URLAudio(), AudioService()
 {
 	TrunkEvent event;
 	unsigned long opts;
-	CT_DEVINFO ctinfo;
 	short wparm;
 	char bparm;
 	const char *cp;
@@ -276,7 +275,7 @@ DialogicTrunk::~DialogicTrunk()
 
 bool DialogicTrunk::Join(DialogicTrunk *trunk)
 {
-	int d1, d2, m1, m2, v2, rtn;
+	int d1, d2, m1, m2, rtn;
 	unsigned flags = MD_PCM | PM_SR8;
 	SC_TSINFO tsinfo1, tsinfo2, rtsinfo;
 	long scts1, scts2, arrayp[32];
@@ -313,8 +312,6 @@ bool DialogicTrunk::Join(DialogicTrunk *trunk)
 		return false;
 	}
 	trunk->enterMutex();
-	tsRelease();
-	trunk->tsRelease();
         nr_scroute(d1, m1, d2, m2, SC_FULLDUP);
         join = trunk;
         join->join = this;
@@ -401,7 +398,6 @@ bool DialogicTrunk::Listen(DialogicTrunk *trunk)
 	}
 
 	HOLD("join")
-	tsRelease();
 
 	switch(data.join.direction)
 	{
@@ -673,9 +669,11 @@ retry:
 	case TRUNK_STOP_DISCONNECT:
 		if(Trunk::flags.onexit)
 			break;
-		trunkSignal(TRUNK_SIGNAL_HANGUP);
-		event->id = TRUNK_STOP_STATE;
-		goto retry;
+		if(trunkSignal(TRUNK_SIGNAL_HANGUP))
+		{
+			event->id = TRUNK_STOP_STATE;
+			goto retry;
+		}
 		break;
 	case TRUNK_SEND_MESSAGE:
 		if(recvEvent(event))
@@ -746,6 +744,7 @@ void DialogicTrunk::exit(void)
 	if(!Trunk::flags.onexit)
 		if(redirect("::exit"))
 		{
+			autoloop(false);
 			Trunk::flags.onexit = true;
 			return;
 		}
@@ -942,7 +941,6 @@ bool DialogicTrunk::answerISDN(void)
 
 bool DialogicTrunk::hangupISDN(void)
 {
-	int state = 0;
 	int rtn;
 	LINEDEV ld;
 
@@ -1073,6 +1071,7 @@ bool DialogicTrunk::dialISDN(char digit[16], int timeout)
 	{
 		makecall_blk.isdn.origination_number_type = ISDN_NOTUSED;
 		makecall_blk.isdn.origination_number_plan = ISDN_NOTUSED;
+		makecall_blk.isdn.origination_sub_number_type = ISDN_NOTUSED;
 		makecall_blk.isdn.origination_phone_number[0] = '\0';
 	}
 	makecall_blk.isdn.facility_feature_service = ISDN_SERVICE;
@@ -1148,6 +1147,10 @@ void DialogicTrunk::playAudio(void)
 		break;
 	case voxADPCM:
 		flags = MD_ADPCM | PM_SR6;
+		break;
+	case pcm16Mono:
+		flags = MD_PCM | PM_SR8;
+		break;
 	}
 	HOLD("play")
 		dx_play(chdev, &iofile, dialogicivr.getTPT(), 
@@ -1197,6 +1200,7 @@ int DialogicTrunk::getCallerId(int mode, unsigned char *cid)
 int DialogicTrunk::getISDNInfo(int mode, unsigned char *cid)
 {
 	int rtn;
+	int mdig;
 	if(!isdn)
 		return getCallerId(mode, cid);
 
@@ -1205,6 +1209,15 @@ int DialogicTrunk::getISDNInfo(int mode, unsigned char *cid)
 		{
 		case MCLASS_DDN:
 			rtn = cc_GetDNIS(crn, (char *)cid);
+			mdig = group->getMinDigits() - strlen((char *)cid);
+			if (mdig > 0) {
+				slog(Slog::levelDebug) << "dx(" << id << "): Getting " << mdig << " more digits" << endl;
+				if (cc_GetMoreDigits(crn, mdig, group->getMDigTimeOut(), EV_SYNC) < 0) {
+					slog(Slog::levelDebug) << "dx(" << id << "): Could not get more digits" << endl;
+				} else {
+					rtn = cc_GetDNIS(crn, (char *)cid + strlen((char *)cid));
+				}
+			}
 			break;
 		case MCLASS_DN:
 			rtn = cc_GetANI(crn, (char *)cid);
@@ -1248,12 +1261,16 @@ bool DialogicTrunk::scrJoin(void)
 	if(!mode)
 		mode = "duplex";
 
+	data.join.hangup = false;
+	data.join.waiting = NULL;
+	data.join.direction = JOIN_FULL;
+
 	if(!stricmp(mode, "receive"))
 		data.join.direction = JOIN_RECV;
 	else if (!stricmp(mode, "transmit"))
 		data.join.direction = JOIN_XMIT;
-	else
-		data.join.direction = JOIN_FULL;
+	else if(!stricmp(mode, "hangup"))
+		data.join.hangup = true;
 
 	if(NULL != cp)
 	{
@@ -1267,11 +1284,21 @@ bool DialogicTrunk::scrJoin(void)
 			data.join.extension = getSymbol(SYM_EXTENSION);
 	}
 	else
-	{
 		data.join.recfn = NULL;
-	}
-	
-	data.join.trunk = trunk;
+
+        mode = getKeyword("count");
+        if(mode)
+                data.join.count = atoi(mode);
+        else
+                data.join.count = 0;
+
+        mode = getKeyword("waitTime");
+        if(mode)
+                data.join.count = getSecTimeout(mode) / 
+			keythreads.getResetDelay() + 1;
+
+	data.join.seq = trunk->getSequence();
+	data.join.waiting = data.join.trunk = trunk;
 	data.join.wakeup = getTimeout("maxTime");
         trunkStep(TRUNK_STEP_JOIN);
         return false;
@@ -1282,6 +1309,7 @@ bool DialogicTrunk::scrWait(void)
 	struct stat ino;
 	char *fn;
 	char *cp;
+	const char *mem = getMember();
 
 	if(NULL != (cp = getKeyword("play")))
 	{
@@ -1306,6 +1334,23 @@ bool DialogicTrunk::scrWait(void)
 				playAudio();
 		}
 	}
+
+	if(mem)
+	{
+		data.join.waiting = NULL;
+		data.join.hangup = false;
+		if(!stricmp(mem, "hangup"))
+			data.join.hangup = true;
+		else if(!stricmp(mem, "parent"))
+			data.join.waiting = dialogicivr.getTrunkId(getSymbol(SYM_PARENT));
+	}
+	else
+	{
+		data.join.waiting = NULL;
+		data.join.hangup = false;
+	}
+
+	data.join.count = 0;
         data.join.trunk = NULL;
         data.join.wakeup = getTimeout("maxTime");
         trunkStep(TRUNK_STEP_JOIN);
