@@ -16,6 +16,11 @@
 
 #include "driver.h"
 
+#ifdef	CCXX_NAMESPACES
+using namespace std;
+namespace ost {
+#endif
+
 bool PhonedevTrunk::answerHandler(TrunkEvent *event)
 {
 	timeout_t wakeup;
@@ -25,22 +30,24 @@ bool PhonedevTrunk::answerHandler(TrunkEvent *event)
 	case TRUNK_LINE_WINK:
 		if(!flags.offhook)
 			return true;
-	case TRUNK_STOP_DISCONNECT:
 	case TRUNK_CPA_DIALTONE:
+		setSymbol(SYM_TONE, "dial");
+	case TRUNK_STOP_DISCONNECT:
 	case TRUNK_TIMER_EXPIRED:
 		flags.dsp = DSP_MODE_INACTIVE;
-		TrunkSignal(TRUNK_SIGNAL_HANGUP);
+		trunkSignal(TRUNK_SIGNAL_HANGUP);
 		handler = &PhonedevTrunk::stepHandler;
 		return true;
 	case TRUNK_ENTER_STATE:
+		setSymbol(SYM_TONE, "none");
 		if(flags.offhook)
 		{
-			TrunkSignal(TRUNK_SIGNAL_STEP);
+			trunkSignal(TRUNK_SIGNAL_STEP);
 			handler = &PhonedevTrunk::stepHandler;
 			return true;
 		}
-		status[id] = 'r';
-		debug->DebugState(this, "answer");
+		status[id] = 'a';
+		enterState("answer");
 	case TRUNK_RINGING_OFF:
 		if(rings > 1)
 			flags.dsp = DSP_MODE_INACTIVE;
@@ -54,10 +61,14 @@ bool PhonedevTrunk::answerHandler(TrunkEvent *event)
 		flags.dsp = DSP_MODE_INACTIVE;
 		setHookState(true);
 		setCPADetect(CPA_DIALTONE, true);
-		TrunkSignal(TRUNK_SIGNAL_STEP);
+		trunkSignal(TRUNK_SIGNAL_STEP);
 		handler = &PhonedevTrunk::stepHandler;
 		status[id] = 'i';
 		return true;
 	}
 	return false;
 }
+
+#ifdef	CCXX_NAMESPACES
+};
+#endif
