@@ -16,8 +16,13 @@
 
 #include "driver.h"
 
+#ifdef	CCXX_NAMESPACES
+namespace ost {
+using namespace ost;
+#endif
+
 PhonedevService::PhonedevService() :
-Thread(NULL, keythreads.priService(), phivr.getStack()), Mutex()
+Thread(keythreads.priService(), phivr.getStack()), Mutex()
 {
 	long opt;
 
@@ -32,18 +37,18 @@ Thread(NULL, keythreads.priService(), phivr.getStack()), Mutex()
 
 PhonedevService::~PhonedevService()
 {
-	Update(0);
-	Terminate();
+	update(0);
+	terminate();
 
 	while(first)
 		delete first;
 }
 
-void PhonedevService::Attach(PhonedevTrunk *trunk)
+void PhonedevService::attach(PhonedevTrunk *trunk)
 {
 	int dev;
 
-	EnterMutex();
+	enterMutex();
 	if(last)
 		last->next = trunk;
 
@@ -57,25 +62,25 @@ void PhonedevService::Attach(PhonedevTrunk *trunk)
 	if(!first)
 	{
 		first = trunk;
-		LeaveMutex();
-		Start();
+		leaveMutex();
+		start();
 	}
 	else
 	{
-		LeaveMutex();
-		Update();
+		leaveMutex();
+		update();
 	}
 	++count;
 }
 
-void PhonedevService::Detach(PhonedevTrunk *trunk)
+void PhonedevService::detach(PhonedevTrunk *trunk)
 {
-	EnterMutex();
+	enterMutex();
 	FD_CLR(trunk->dev, &connect);
 	if(trunk == first && trunk == last)
 	{
 		first = last = NULL;
-		LeaveMutex();
+		leaveMutex();
 		return;
 	}
 	if(trunk->prev)
@@ -90,17 +95,17 @@ void PhonedevService::Detach(PhonedevTrunk *trunk)
 	if(trunk == last)
 		last = trunk->prev;
 
-	LeaveMutex();
-	Update();
+	leaveMutex();
+	update();
 	--count;
 }		
 
-void PhonedevService::Update(unsigned char flag)
+void PhonedevService::update(unsigned char flag)
 {
 	::write(iosync[1], (char *)&flag, 1);
 }
 
-void PhonedevService::Run(void)
+void PhonedevService::run(void)
 {
 	TrunkEvent event;
 	struct timeval timeout, *tvp;
@@ -114,7 +119,7 @@ void PhonedevService::Run(void)
 
 	FD_ZERO(&err);
 
-	setCancel(THREAD_CANCEL_DEFERRED);
+	setCancel(cancelDeferred);
 	for(;;)
 	{
 		timer = ~0;
@@ -122,23 +127,23 @@ void PhonedevService::Run(void)
 		{
 			if(buf && buf != 0xff)
 			{ 
-				debug->DebugService(driver->getTrunkPort(buf - 1), "notify");
-				phivr.Notify(buf);
+				debug->debugService(driver->getTrunkPort(buf - 1), "notify");
+				phivr.notify(buf);
 				continue;
 			}
 			if(buf)
 			{
-				debug->DebugService(NULL, "updating");
+				debug->debugService(NULL, "updating");
 				continue;
 			}
-			debug->DebugService(NULL, "exiting");
-			setCancel(THREAD_CANCEL_IMMEDIATE);
-			Sleep(~0);
-			Exit();
+			debug->debugService(NULL, "exiting");
+			setCancel(cancelDeferred);
+			Thread::sleep(~0);
+			exit();
 		}
 
-		Yield();
-		EnterMutex();
+		yield();
+		enterMutex();
 		trunk = first;
 		while(trunk)
 		{
@@ -146,7 +151,7 @@ void PhonedevService::Run(void)
 
 			if(FD_ISSET(dev, &err))
 			{
-				debug->DebugService(trunk, "event");
+				debug->debugService(trunk, "event");
 				trunk->getEvents();
 			}
 
@@ -158,7 +163,7 @@ retry:
 
 			if(!expires)
 			{
-				debug->DebugService(trunk, "expires");
+				debug->debugService(trunk, "expires");
 				event.id = TRUNK_TIMER_EXPIRED;
 				trunk->endTimer();
 				trunk->postEvent(&event);
@@ -167,7 +172,7 @@ retry:
 
 			trunk = trunk->next;
 		}
-		LeaveMutex();
+		leaveMutex();
 		memcpy(&err, &connect, sizeof(err));
 		FD_ZERO(&inp);
 		FD_SET(iosync[0], &inp);
@@ -180,9 +185,12 @@ retry:
 			timeout.tv_usec = (timer % 1000) * 1000;
 		}
 		sprintf(tmp, "delay %d", timer);
-		debug->DebugService(NULL, tmp);
+		debug->debugService(NULL, tmp);
 		select(hiwater, &inp, NULL, &err, tvp);
-		debug->DebugService(NULL, "wakeup");
+		debug->debugService(NULL, "wakeup");
 	}
 }
 
+#ifdef	CCXX_NAMESPACES
+};
+#endif
