@@ -16,21 +16,34 @@
 
 #include "gui.h"
 
+#ifdef	CCXX_NAMESPACES
+namespace ost {
+using namespace std;
+#endif
+
 GUITabs::GUITabs(FXComposite *f) :
 FXTabBook(f, NULL, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)
 {
-	int item;
-
 	mkitem(0, "&Info");
-	new TabInfo(this);
-	mkitem(1, "&Nodes");
-	new FXHorizontalFrame(this, FRAME_THICK | FRAME_RAISED);
+	info = new TabInfo(this);
+	mkitem(1, "&Calls");
+	calls = new TabCalls(this);
 	mkitem(2, "&Groups");
-	new FXHorizontalFrame(this, FRAME_THICK | FRAME_RAISED);
+	groups = new TabGroups(this);
 	mkitem(3, "&Lines");
-	new FXHorizontalFrame(this, FRAME_THICK | FRAME_RAISED);
+	lines = new TabLines(this);
 }
 
+void GUITabs::onState(Trunk *trunk, const char *state)
+{
+	calls->onState(trunk, state);
+	lines->onState(trunk, state);
+}
+
+void GUITabs::onStep(Trunk *trunk, Script::Line *line)
+{
+	lines->onStep(trunk, line);
+}
 
 void GUITabs::mkitem(int id, FXchar *header)
 {
@@ -38,3 +51,6 @@ void GUITabs::mkitem(int id, FXchar *header)
 	items[id]->setFont(gui.getTitleFont());
 }
 
+#ifdef	CCXX_NAMESPACES
+};
+#endif
