@@ -348,10 +348,10 @@ static bool saylocal(strstream *s, double v, bool orflag)
 	if(cvt)
 	{
 		v *= atof(cvt);
-		sprintf(tbuf, "%d", v);
+		sprintf(tbuf, "%f", v);
 	}
 	else
-		sprintf(tbuf, "%d", v);	
+		sprintf(tbuf, "%f", v);	
 	if(!v)
 		return false;
 	*s << ",or";
@@ -465,11 +465,7 @@ char *ItalianTranslator::speak(Trunk *trunk)
 	bool skip = false;
 	long lastnum = 0;
 	char *text;
-#ifdef	SCRIPT_LOCAL_DEFINE
 	ScriptImage *cmd = trunk->getImage();
-#else
-	ScriptCommand *cmd = trunk->getCommand();
-#endif
 
 	while(NULL != (rule = trunk->getValue(NULL)))
 	{
@@ -479,7 +475,6 @@ char *ItalianTranslator::speak(Trunk *trunk)
 			continue;
 		}
 		phrase = getLast(rule + 1);
-#ifdef	SCRIPT_PREPROCESSOR_OVERRIDE
 		if(!phrase)
 		{
 			snprintf(rules, 256, "_italian_%s", rule + 1);
@@ -490,7 +485,6 @@ char *ItalianTranslator::speak(Trunk *trunk)
 			snprintf(rules, 256, "_all_%s", rule + 1);
 			phrase = cmd->getLast(rules);
 		}
-#endif
 		if(!phrase)
 			phrase = rule;
 
