@@ -107,6 +107,13 @@ typedef enum
 	JOIN_FULL
 } joinmode_t;
 
+typedef enum
+{
+	PULSE_DIALER,
+	DTMF_DIALER,
+	MF_DIALER
+} dialmode_t;
+
 typedef enum {
 	MODULE_GENERIC,
 	MODULE_DELIVERY,
@@ -118,11 +125,14 @@ typedef enum {
 	MODULE_URL,
 	MODULE_NET,
 	MODULE_REPLY,
-	MODULE_ASR,
-	MODULE_TTS,
 #ifdef	XML_SCRIPTS
 	MODULE_XML,
 #endif
+	MODULE_SWITCH,
+	MODULE_THREAD,
+	MODULE_PLAY,
+	MODULE_RECORD,
+	MODULE_SQL,
 	MODULE_ANY
 }	modtype_t;
 
@@ -139,6 +149,7 @@ typedef enum {
 	TRUNK_STEP_RECORD,
 	TRUNK_STEP_TONE,
 	TRUNK_STEP_DIALXFER,
+	TRUNK_STEP_SOFTDIAL,
 	TRUNK_STEP_FLASH,
 	TRUNK_STEP_JOIN,
 	TRUNK_STEP_RTP,
@@ -146,6 +157,12 @@ typedef enum {
 	TRUNK_STEP_DETECT,
 	TRUNK_STEP_REQUIRES,
 	TRUNK_STEP_LOADER,
+	TRUNK_STEP_TRANSFER,	// pbx internal transfer
+	TRUNK_STEP_INTERCOM,	// pbx internal intercom
+	TRUNK_STEP_PICKUP,	// pbx internal line pickup
+	TRUNK_STEP_THREAD,
+	TRUNK_STEP_SENDFAX,
+	TRUNK_STEP_RECVFAX,
 	TRUNK_STEP_EXIT = TRUNK_STEP_HANGUP,
 	TRUNK_STEP_DRIVER
 } trunkstep_t;
@@ -153,42 +170,57 @@ typedef enum {
 typedef	enum {
 	// notify script from state handler
 	TRUNK_SIGNAL_STEP = 0,
+
 	TRUNK_SIGNAL_EXIT,
 	TRUNK_SIGNAL_HANGUP=TRUNK_SIGNAL_EXIT,
 	TRUNK_SIGNAL_ERROR,
 	TRUNK_SIGNAL_TIMEOUT,
 	TRUNK_SIGNAL_DTMF,
+
 	TRUNK_SIGNAL_0,
 	TRUNK_SIGNAL_1,
 	TRUNK_SIGNAL_2,
 	TRUNK_SIGNAL_3,
+
 	TRUNK_SIGNAL_4,
 	TRUNK_SIGNAL_5,
 	TRUNK_SIGNAL_6,
 	TRUNK_SIGNAL_7,
+
 	TRUNK_SIGNAL_8,
 	TRUNK_SIGNAL_9,
 	TRUNK_SIGNAL_STAR,
 	TRUNK_SIGNAL_POUND,
+
 	TRUNK_SIGNAL_A,
+	TRUNK_SIGNAL_OVERRIDE = TRUNK_SIGNAL_A,
 	TRUNK_SIGNAL_B,
+	TRUNK_SIGNAL_FLASH = TRUNK_SIGNAL_B,
 	TRUNK_SIGNAL_C,
+	TRUNK_SIGNAL_IMMEDIATE = TRUNK_SIGNAL_C,
 	TRUNK_SIGNAL_D,
+	TRUNK_SIGNAL_PRIORITY = TRUNK_SIGNAL_D,
+
 	TRUNK_SIGNAL_SILENCE,
 	TRUNK_SIGNAL_BUSY,
 	TRUNK_SIGNAL_CANCEL,
 	TRUNK_SIGNAL_FAIL = TRUNK_SIGNAL_CANCEL,
 	TRUNK_SIGNAL_INVALID = TRUNK_SIGNAL_CANCEL,
 	TRUNK_SIGNAL_SIGNAL,
+
 	TRUNK_SIGNAL_NOANSWER,
 	TRUNK_SIGNAL_RING,
+	TRUNK_SIGNAL_ANSWER = TRUNK_SIGNAL_RING,
+	TRUNK_SIGNAL_PICKUP = TRUNK_SIGNAL_RING,
 	TRUNK_SIGNAL_TONE,
 	TRUNK_SIGNAL_EVENT,
+
 	TRUNK_SIGNAL_TIME,
 	TRUNK_SIGNAL_MAXTIME = TRUNK_SIGNAL_TIME,
 	TRUNK_SIGNAL_CHILD,
 	TRUNK_SIGNAL_DRIVER,
-	TRUNK_SIGNAL_GOTO = 65,
+
+	TRUNK_SIGNAL_GOTO = 65
 }	trunksignal_t;
 
 typedef enum {
@@ -272,6 +304,10 @@ typedef enum {
 	TRUNK_CALL_NSI,		// ISDN call nsi message
 	TRUNK_CALL_RINGING,	// digital T1 incoming call
 	TRUNK_CALL_DISCONNECT,	// digital T1 circuit break
+	TRUNK_DEVICE_OPEN,	// device open
+	TRUNK_DEVICE_CLOSE,	// device close
+	TRUNK_DEVICE_BLOCKED,	// channel blocked
+	TRUNK_DEVICE_UNBLOCKED,	// channel unblocked
 
 	// basic audio processing events
 
@@ -301,6 +337,22 @@ typedef enum {
 	TRUNK_CPA_GRUNT,
 	TRUNK_CPA_REORDER,
 	TRUNK_DSP_READY,	// dsp resource became available
+	TRUNK_CPA_STOPPED,
+
+	// basic station processing event extensions
+
+	TRUNK_START_RINGING = 600,	// ring for incoming lines
+	TRUNK_START_TRANSFER,		// transfer mode ring
+	TRUNK_START_INTERCOM,		// intercom mode ring
+	TRUNK_START_RECALL,		// transfer recall mode ring
+	TRUNK_START_DIALING,		// initiate outgoing call
+	TRUNK_STOP_RINGING,		// cancel a ring request
+	TRUNK_STATION_OFFHOOK,
+	TRUNK_STATION_ONHOOK,
+	TRUNK_STATION_FLASH,
+	TRUNK_STATION_ANSWER,		// answering ringing port
+	TRUNK_STATION_PICKUP,		// pickup foreign port
+	TRUNK_STATION_CONNECT,		// autoanswer mode
 
 	// driver specific events and anomolies
 
@@ -344,7 +396,16 @@ typedef enum
 	STAT_MAX_OUTGOING,
 	STAT_TOT_INCOMING,
 	STAT_TOT_OUTGOING,
-	STAT_ACTIVE_CALLS
+	STAT_ACTIVE_CALLS,
+	STAT_NOW_INCOMING,
+	STAT_NOW_OUTGOING,
+	STAT_SYS_INCOMING,
+	STAT_SYS_OUTGOING,
+	STAT_SYS_UPTIME,
+	STAT_SYS_ACTIVITY,
+	STAT_CURRENT_CALLS,
+	STAT_CMAX_INCOMING,
+	STAT_CMAX_OUTGOING
 } statitem_t;
 
 typedef	enum
@@ -352,7 +413,10 @@ typedef	enum
 	PLAY_MODE_NORMAL,
 	PLAY_MODE_ONE,
 	PLAY_MODE_ANY,
-	PLAY_MODE_TEMP
+	PLAY_MODE_TEMP,
+	PLAY_MODE_TEXT,
+	PLAY_MODE_FILE,
+	PLAY_MODE_MOH
 } playmode_t;
 
 typedef enum
@@ -368,6 +432,7 @@ typedef enum
 #define	TRUNK_CAP_DATA		0x00000010
 #define	TRUNK_CAP_TTS		0x00000020
 #define	TRUNK_CAP_ASR		0x00000040
+#define	TRUNK_CAP_STATION	0x00000080
 
 typedef	struct
 {
@@ -377,14 +442,30 @@ typedef	struct
 	void *data;
 }	execdata_t;
 
+typedef enum
+{
+	SPEED_FAST,
+	SPEED_SLOW,
+	SPEED_NORMAL
+}	trunkspeed_t;
+
 typedef	union
 {
 	struct
 	{
-		int rings;
-		int timeout;
+		unsigned rings;
+		timeout_t timeout;
+		const char *transfer;
+		Trunk *intercom;
+		const char *station;	// if answer to fax tone, station id
+		const char *fax;	// fax branch script vector
 	}	answer;
 	struct
+	{
+		char pathname[256];
+		const char *station;	// fax station id
+	}	fax;
+	struct
 	{
 		char list[256];
 		char *name;
@@ -396,13 +477,16 @@ typedef	union
 		timeout_t timeout, maxtime;
 		unsigned repeat;
 		unsigned volume;
-		float gain;
+		float gain, pitch;
+		trunkspeed_t speed;
 		const char *voice;
 		const char *text;
+		const char *cache;
+		bool lock;
 	}	play;
 	struct
 	{
-		char *name;
+		char *name, *save;
 		const char *encoding;
 		const char *annotation;
 		const char *extension;
@@ -418,17 +502,23 @@ typedef	union
 		short frames;
 		bool append;
 		bool info;
+		char filepath[65];
+		char savepath[65];
+		char altinfo[128];
 	}	record;
 	struct
 	{
 		char digits[65];
 		char *digit;
              	char *callingdigit;
-		timeout_t interdigit;
 		bool exit;
+		dialmode_t dialer;
+		timeout_t interdigit;
+		timeout_t digittimer;
 		timeout_t timeout;
 		timeout_t offhook;
 		timeout_t onhook;
+		unsigned pulsecount;
 	}	dialxfer;
 	struct
 	{
@@ -453,6 +543,8 @@ typedef	union
 		phTone *tone;
 		unsigned freq1, freq2;
 		int ampl1, ampl2;
+		Trunk *dialing;
+		bool recall;
 	}	tone;
 	struct
 	{
@@ -462,35 +554,39 @@ typedef	union
 	}	send;
 	struct
 	{
-		timeout_t wakeup;
+		timeout_t wakeup, maxwait;
+		bool hangup;
 		joinmode_t direction;
-		Trunk *trunk;
+		Trunk *trunk, *waiting;
 		phTone *tone;
 		float inpgain, outgain;
 		char *recfn;
 		const char *encoding;
 		const char *annotation;
 		const char *extension;
+		unsigned count;
+		unsigned seq;
+		trunkevent_t reason;
 	}	join;
 	struct
-	{
-		struct in_addr addr;
-		unsigned short bind, port, term;
-		Audio::Encoding codec;
-	}	rtp;
-	struct
 	{
 		TrunkImage *image;
 		const char *url;
 		const char *section;
 		const char *parent;
 		const char *fail;
+		const char *database;
 		char **vars;
-		bool post, attach;
+		bool post, attach, gosub;
 		timeout_t timeout;
 		char userid[64];
 		char filepath[256];
 	}	load;
+	struct
+	{
+		Trunk *answer;
+		const char *transfer;
+	}	intercom;
 }	trunkdata_t;
 
 typedef struct
@@ -529,8 +625,15 @@ typedef struct
 			bool result;
 			char *data;
 		} lookup;
+		struct
+		{
+			unsigned tid;
+			const char *transfer;
+		} intercom;
+		trunkevent_t reason;
 		unsigned span;
 		unsigned card;
+		unsigned tid;
 		bool ok;
 		int status;
 		Trunk *trunk;
@@ -550,10 +653,13 @@ typedef	struct {
 	time_t	update;
 	char name[16];
 	struct in_addr addr;
+	unsigned long uptime;
+	unsigned long calls;
 	unsigned char version;
 	unsigned char buddies;
-	unsigned char ports;
-	char stat[255];
+	unsigned char spansize;
+	unsigned short ports;
+	char stat[840];
 }	statnode_t;
 
 #pragma pack()
@@ -563,6 +669,8 @@ typedef	struct {
 */
 
 bool getLogical(const char *string);
+timeout_t getSecTimeout(const char *string);
+timeout_t getMSTimeout(const char *string);
 
 /**
  * A call statistic class is used to manipulate call stats with a mutex
@@ -574,6 +682,9 @@ bool getLogical(const char *string);
 class CallStat : public Mutex
 {
 protected:
+	static unsigned long upincoming, upoutgoing;
+	static time_t uptime;
+
 	int capacity;
 	struct
 	{
@@ -606,6 +717,13 @@ public:
 	 */
 	long getStat(statitem_t item);
 
+	/**
+	 * Get a stat record at once.
+	 *
+	 * @param pointer to stats to copy.
+	 */
+	void getStat(unsigned long *save);
+
 	/**
 	 * inc active incoming call count.
 	 */
@@ -731,6 +849,22 @@ public:
 	KeyHandlers();
 };
 
+/**
+ * Load /etc/bayonne [voices] to select and map voice libraries to
+ * different translators.
+ *
+ * @author David Sugar <dyfet@ostel.com>
+ * @short Load interpreter imports to use.
+ */
+class KeyVoices : public Keydata
+{
+public:
+	/**
+	 * Initialize keyimports data.
+	 */
+	KeyVoices();
+};
+
 /**
  * Load /etc/bayonne [imports] to provide interpreter pre-loading of
  * external modules.  This speeds startup.
@@ -741,12 +875,13 @@ public:
 class KeyImports : public Keydata
 {
 public:
-	/**
-	 * Initialize keyimports data.
-	 */
-	KeyImports();
+        /**
+         * Initialize keyimports data.
+         */
+        KeyImports();
 };
 
+
 /**
  * Load /etc/bayoone [paths] key value pairs.  Has internal defaults
  * if section or file is missing.
@@ -820,7 +955,7 @@ public:
 	 * Get the pre-cache directory.
 	 */
 	inline const char *getCache(void)
-		{return getLast("precache");};
+		{return getLast("cache");};
 
 	/**
 	 * Set auto-location prefix.
@@ -996,6 +1131,22 @@ public:
 	 */
 	inline int getPageSize(void)
 		{return atoi(getLast("page"));};
+
+	/**
+	 * Get maximum users.
+	 *
+	 * @return maximum users.
+	 */
+	inline size_t getUserCount(void)
+		{return atoi(getLast("users"));};
+
+	/**
+	 * Get maximum preferences per user.
+	 *
+	 * @return maximum preferences.
+	 */
+	inline size_t getPrefCount(void)
+		{return atoi(getLast("prefs"));};
 };
 
 /**
@@ -1137,6 +1288,14 @@ public:
 	inline int priNetwork(void)
 		{return atoi(getLast("network"));};
 
+	/**
+	 * Get the relative priority for switch integration module.
+	 *
+	 * @return priority for switch integration.
+	 */
+	inline int priSwitch(void)
+		{return atoi(getLast("switch"));};
+
 #ifdef	XML_SCRIPTS
 	/**
 	 * Get the relative priority of the XML parser thread.
@@ -1366,16 +1525,16 @@ public:
 	 *
 	 * @return number of milliseconds of caller id.
 	 */
-	inline unsigned getCallerid(void)
-		{return atoi(getLast("callerid"));};
+	inline timeout_t getCallerid(void)
+		{return getMSTimeout(getLast("callerid"));};
 
 	/**
 	 * Get pickup gaurd time for this trunk group.
 	 *
 	 * @return number of milliseconds for call pickup.
 	 */
-	inline unsigned getPickup(void)
-		{return atoi(getLast("pickup"));};
+	inline timeout_t getPickup(void)
+		{return getMSTimeout(getLast("pickup"));};
 
 	/**
 	 * Get handling for trunk group "pending requests".
@@ -1415,15 +1574,15 @@ public:
 	 * @return ready timer in milliseconds.
 	 */
 	inline timeout_t getReady(void)
-		{return atol(getLast("ready"));};
+		{return getMSTimeout(getLast("ready"));};
 	
 	/**
 	 * Get the initial idle timer for this group.
 	 *
 	 * @return idle time initial.
 	 */
-	inline unsigned getIdleTime(void)
-		{return atoi(getLast("idletime"));};
+	inline timeout_t getIdleTime(void)
+		{return getMSTimeout(getLast("idletime"));};
 
 	/**
 	 * Get the trunk seizure timer for dialtone detect.
@@ -1441,13 +1600,29 @@ public:
 	inline unsigned getRingTime(void)
 		{return atoi(getLast("ringtime"));};
 
+	/**
+	 * Get call progress minimum digits
+	 *
+	 * @return number of digits (only for dialogic drivers).
+	 */
+	inline unsigned getMinDigits(void)
+		{return atoi(getLast("mindigits"));};
+
+	/**
+	 * Get call progress time out to get more digits
+	 *
+	 * @return Timeout in seconds
+	 */
+	inline unsigned getMDigTimeOut(void)
+		{return atoi(getLast("mdigtimeout"));};
+
 	/**
 	 * Get disconnect gaurd time before answering.
 	 *
 	 * @return gaurd time in milliseconds.
 	 */
-	inline int getHangup(void)
-		{return atoi(getLast("hangup"));};
+	inline timeout_t getHangup(void)
+		{return getMSTimeout(getLast("hangup"));};
 
 	/**
 	 * Get default hook flash time for this trunk group.
@@ -1455,7 +1630,7 @@ public:
 	 * @return hookflash time in milliseconds.
 	 */
 	inline timeout_t getFlash(void)
-		{return atol(getLast("flash"));};
+		{return getMSTimeout(getLast("flash"));};
 
 	/**
 	 * Get dialtone wait time for this trunk group.
@@ -1463,7 +1638,7 @@ public:
 	 * @return dialtone wait time in milliseconds.
 	 */
 	inline timeout_t getDialtone(void)
-		{return atol(getLast("dialtone"));};
+		{return getMSTimeout(getLast("dialtone"));};
 
 	/**
 	 * Get dialing speed in milliseconds.
@@ -1471,7 +1646,7 @@ public:
 	 * @return dialspeed in milliseconds.
 	 */
 	inline timeout_t getDialspeed(void)
-		{return atol(getLast("dialspeed"));};
+		{return getMSTimeout(getLast("dialspeed"));};
 
 	/**
 	 * Get the telephone number associated with this trunk group
@@ -1518,7 +1693,15 @@ public:
 	 *
 	 * @return trunk group object if found.
 	 */
-	friend TrunkGroup *getGroup(const char *name);
+	friend TrunkGroup *getGroup(const char *name = NULL);
+
+	/**
+	 * Get next group record.
+	 *
+	 * @return next group link.
+	 */
+	inline TrunkGroup *getNext(void)
+		{return next;};
 };
 
 /**
@@ -1549,6 +1732,22 @@ public:
 	inline const char *getNode(void)
 		{return getLast("node");};
 
+	/**
+	 * Return default script attach login state.
+	 *
+	 * @return login id.
+	 */
+	inline const char *getLogin(void)
+		{return getLast("login");};
+
+	/**
+	 * Return policy search order.
+	 *
+	 * @return policy search order.
+	 */
+	inline const char *getPolicyOrder(void)
+		{return getLast("policy");};
+
 	/**
 	 * Get the remote access password.
 	 *
@@ -1689,6 +1888,17 @@ public:
 	 */
 	void loadSwitch(void);
 
+	/**
+	 * Attempt to load an optional text to speech plugin module for
+	 * GNU Bayonne.
+	 */
+	void loadTTS(void);
+
+	/**
+	 * Attempt to load a sql query module for Bayonne.
+	 */
+	void loadSQL(void);
+
 	/**
 	 * Load automatic application extensions module.
 	 */
@@ -1806,7 +2016,6 @@ protected:
 	 */
 	void scanDir2(char *path);
 
-#ifdef	SCRIPT_PREPROCESSOR_OVERRIDE
         /**
          * used to create dialect specific pre-precessor directives.  A
 	 * virtual in ccScript 2.1.1, useless otherwise...
@@ -1816,7 +2025,6 @@ protected:
          * @param script object being built
          */
         bool preProcess(const char *directive, Name *script);
-#endif
 
 public:
 	/**
@@ -1993,7 +2201,7 @@ protected:
 	 * @param default if not found.
 	 */
 	const char *getAttribute(const char *key, const char *value = NULL);
-	
+
 public:
 	/**
 	 * Construct a new script image, normally done in the plugin.
@@ -2020,7 +2228,7 @@ public:
 	 * @param true on success, false on failure.
 	 * @param pointer to collected data structure.
 	 */
-	virtual bool loader(trunkdata_t *data) = 0;
+	virtual bool loader(Trunk *trk, trunkdata_t *data) = 0;
 
 	/**
 	 * Access to the purge method of the script image.
@@ -2059,6 +2267,7 @@ private:
 	friend class Fifo;
 	friend class Driver;
 	friend class Module;
+	friend class RPCTransaction;
 
 	typedef union
 	{
@@ -2072,12 +2281,16 @@ private:
         	char data[sizeof(Symbol) + MAX_DIGITS];
 	}       Digit;
 
+protected:
 	phTone *tonetmp;
+
+private:
 	unsigned member;
 	Line	*cdrv;
 	int cdrc;
 #ifdef	XML_SCRIPTS
 	TrunkImage *altimage;
+	Module *altmodule;
 #endif
 	char apppath[64];
 
@@ -2085,19 +2298,52 @@ private:
 	bool scrConfig(void);
 	bool scrSend(void);
 	bool scrStart(void);
+#ifdef	HAVE_TGI
 	bool scrLibexec(void);
+#endif
+	bool scrControl(void);
 	bool scrHangup(void);
 	bool scrDebug(void);
 	bool scrAlog(void);
 	bool scrAudit(void);
 	bool scrSleep(void);
+
+#ifdef	SCRIPT_IF_OVERRIDE
+public:
+	static bool hasVoice(ScriptInterp *interp, const char *v);
+	static bool hasAppVoice(ScriptInterp *interp, const char *v);
+	static bool hasSysVoice(ScriptInterp *interp, const char *v);
+	static bool hasAltVoice(ScriptInterp *interp, const char *v);
+	static bool hasGroup(ScriptInterp *interp, const char *v);
+	static bool hasPlugin(ScriptInterp *interp, const char *v);
+	static bool isNode(ScriptInterp *interp, const char *v);
+	static bool isService(ScriptInterp *interp, const char *v);
+	static bool ifDTMF(ScriptInterp *interp, const char *v);
+	static bool ifFeature(ScriptInterp *interp, const char *v);
+	static bool isExtension(ScriptInterp *interp, const char *v);
+	static bool isStation(ScriptInterp *interp, const char *v);
+	static bool isVirtual(ScriptInterp *interp, const char *v);
+	static bool isHunt(ScriptInterp *interp, const char *v);
+	static bool isActiveUser(ScriptInterp *interp, const char *v);
+	static bool isDnd(ScriptInterp *interp, const char *v);
+#endif
+
 protected:
+	bool scrCommit(void);
+	bool scrChange(void);
+	bool scrPassword(void);
+	bool scrLogin(void);
+	bool scrLogout(void);
 	bool scrSync(void);
+	bool scrAnswer(void);
+	bool scrDial(void);
+	bool scrTransfer(void);
+	bool scrHold(void);
 	bool scrOptions(void);
+	bool scrRedirect(void);
 
 private:
 	bool scrTone(void);
-	bool scrAnswer(void);
 	bool scrAccept(void);
 	bool scrReject(void);
 	bool scrCollect(void);
@@ -2106,19 +2352,20 @@ private:
 	bool scrSay(void);
 	bool scrCleardigits(void);
 	bool scrAssign(void);
+	bool scrAltPlay(void);
+	bool scrAltSpeak(void);
 	bool scrPlay(void);
+	bool scrSendFax(void);
+	bool scrRecvFax(void);
 #ifdef	XML_SCRIPTS
 	bool scrLoad(void);
 #endif
+#ifdef	HAVE_TGI
 	bool scrCopy(void);
-	bool scrExists(void);
+#endif
 	bool scrMove(void);
 	bool scrErase(void);
 	bool scrRecord(void);
-	bool scrDial(void);
-	bool scrDTMF(void);
-	bool scrTransfer(void);
-	bool scrHold(void);
 	bool scrSpeak(void);
 	bool scrModule(void);
 	bool scrDummy(void);
@@ -2128,35 +2375,39 @@ private:
 	bool scrBusy(void);
 	bool scrExamine(void);
 	bool scrService(void);
+	bool scrUserinfo(void);
+	bool scrHuntinfo(void);
+	bool scrStatinfo(void);
+
+protected:
 
 #ifdef	XML_SCRIPTS
 	Name *getScriptImage(const char *name);
 #endif
 
-#ifdef	SCRIPT_EXCLUSIVE_OVERRIDE
 	void setExclusive(bool enable)
 		{globals.setExclusive(enable);};
-#endif
 
-protected:
 	Number numbers[6];
 	static ScriptSymbol globals;
 	static char digit[16];
 	ScriptInterface *script;
 	TrunkGroup *group;
-	int id, span;
+	int id;
+	unsigned span;
 	time_t starttime, idletime, synctimer, exittimer;
 
-	unsigned seq;
+	volatile unsigned seq;
 	int idle_timer;
-	int rings;
-	int digits;
+	unsigned rings, digits;
 
 	Service *thread;
 	trunkdata_t data;
 	execdata_t tgi;
 	Digit dtmf;
 	char buffer[65];
+	char extNumber[5];
+	bool dialgroup[10];
 
 	struct
 	{
@@ -2173,6 +2424,9 @@ protected:
 		trunkmode_t trunk: 2;
 		dspmode_t dsp: 4;
 		dtmfmode_t digits: 2;
+		bool dnd : 1;
+		bool cid : 1;	// captured cid state
+		bool sent : 1;	// cid info sent
 	} flags;
 
 	/**
@@ -2217,6 +2471,19 @@ protected:
  	 */
 	void commit(Symbol *sym);
 
+public:
+	/**
+	 * Flag if admin user.
+	 */
+	bool isAdmin(void);
+
+	/**
+	 * Get a fax station identifier for a script.
+	 *
+	 * @return pointer to station id string.
+	 */
+	const char *getStation(void);
+
 	/**
 	 * Get a "timeout" option.  This is like getValue, however
 	 * the default timeout supplied is from the constant table,
@@ -2228,6 +2495,15 @@ protected:
 	 */
 	timeout_t getTimeout(const char *keyword = NULL);
 
+	/**
+	 * Extract an extension reference, either directly, or from
+	 * an encoded global/local session identifier.
+	 *
+	 * @return extension number or NULL if invalid.
+	 * @param extension number or session id.
+	 */
+	static const char *getExtReference(const char *ref);
+
 	/**
 	 * Get a "interdigit" timeout option.  This is like getValue,
 	 * however the interdigit value supplied is from the const table.
@@ -2243,6 +2519,7 @@ protected:
 	 */
 	unsigned short getDigitMask(const char *keyword = NULL);
 
+protected:
 	/**
 	 * Notify the script subsystem of a completion event.
 	 *
@@ -2379,6 +2656,21 @@ protected:
 
 	Trunk(int port, int card = 0, int span = 0);
 public:
+	/**
+	 * Get the timeslot port number.
+	 *
+	 * @return driver port.
+	 */
+	unsigned getId(void)
+		{return id;};
+	/**
+	 * Get the trunk sequence number.
+	 *
+	 * @return sequence.
+	 */
+	inline unsigned getSequence(void)
+		{return seq;};
+
 	/**
 	 * Set the sleep save marker for sleeping tasks.
 	 *
@@ -2403,6 +2695,41 @@ public:
 	static inline ScriptSymbol *getGlobals(void)
 		{return &globals;};
 
+	/**
+	 * Load user preferences.
+	 */
+	static void load(const char *id);
+
+	/**
+	 * Load line preferences.
+	 */
+	static void loadPref(const char *id, const char *prefix, const char *dir);
+
+	/**
+	 * Save user preferences.
+	 */
+	static void save(const char *id);
+
+	/**
+	 * Save line preferences.
+	 */
+	static void savePref(const char *id, const char *prefix, const char *dir);
+
+	/**
+	 * Sync session settings.
+	 */
+	static void sync(bool force = false);
+
+	/**
+	 * Init password database.
+	 */
+	static void initPassword(void);
+
+	/**
+	 * Init trunk line database.
+	 */
+	static void initLines(void);
+
 	/**
 	 * Get the device logical name number.
 	 *
@@ -2441,15 +2768,7 @@ public:
 	 */
 	bool isReady(void);
 
-	/**
-	 * Perform a libexec operation on behalf of the current
-	 * trunk.
-	 *
-	 * @param command string to pass.
-	 * @param timeout for command in seconds.
-	 */
-	void libexec(char **argv, unsigned timeout);
-
+#ifdef	HAVE_TGI
 	/**
 	 * Perform a gateway TTS operation on behalf of the current
 	 * trunk.
@@ -2458,6 +2777,7 @@ public:
 	 * @param mode argument.
 	 */
 	void libtts(const char *msg, ttsmode_t mode);
+#endif
 
 	/**
 	 * Make getOnce() into a public.
@@ -2487,6 +2807,14 @@ public:
 	 */
 	inline ScriptCommand *getCommand(void)
 		{return ScriptInterp::getCommand();};
+
+	/**
+	 * Fetch the current trunk mode flag.
+	 *
+	 * @return trunk call mode.
+	 */
+	inline trunkmode_t getTrunkMode(void)
+		{return flags.trunk;};
 };
 
 /**
@@ -2498,11 +2826,17 @@ public:
  * @author David Sugar <dyfet@ostel.com>
  * @short Bayonne fifo command executive.
  */
-class Fifo : public Mutex
+class Fifo : public Mutex, public Script
 {
 protected:
 	char schedule[33];
 
+	bool assignRing(char **args);
+	bool assignDial(char **args);
+	bool sendEvent(char **args);
+	bool clearRing(char **args);
+	bool clearDial(char **args);
+	bool clearRDGroup(char **args);
 	bool exitPid(char **args);
 	bool waitPid(char **args);
 	bool setSymbol(char **argv);
@@ -2510,6 +2844,7 @@ protected:
 
 	bool login(char **argv);
 	bool startScript(char **argv);
+	bool testScript(char **argv);
 	bool ringScript(char **argv);
 	bool redirectScript(char **argv);
 	bool setSpan(char **argv);
@@ -2525,6 +2860,9 @@ protected:
 	bool mapFiles(char **argv);
 	bool submit(char **argv);
 	bool reload(char **argv);
+	bool saveId(char **argv);
+	bool createId(char **argv);
+	bool deleteId(char **argv);
 
 public:
 	/**
@@ -2574,7 +2912,7 @@ protected:
 	/**
 	 * Post a cdr line as built by "cdr"
 	 *
-	 * @oaram detail line.
+	 * @param detail line.
 	 */
 	virtual void reportCDR(Trunk *trunk, char *detail)
 		{reportAudit(trunk, detail);};
@@ -2624,10 +2962,12 @@ protected:
 	friend class TrunkGroup;
 	friend class Trunk;
 
-	TrunkGroup **groups, *spans[MAX_SPANS], *cards[MAX_CARDS];
+	TrunkGroup **groups, *spans[MAX_SPANS], *cards[MAX_CARDS], *stacards[MAX_CARDS];
 	char *status;
 	bool active;
 	unsigned portCount, downCount;
+	unsigned extCount, trkCount, tieCount, numCount;
+	Trunk **extIndex;
 	volatile unsigned idleCount;
 
 public:
@@ -2635,6 +2975,10 @@ public:
 		capPSTN = 0x0002,
 		capJoin = 0x00004,
 		capSpans = 0x0008,
+		capSpeed = 0x0010,	// driver can do rate adj
+		capPitch = 0x0020,	// driver can pitch adj
+		capGain = 0x0040,	// driver can gain adj
+		capSwitch = 0x2000,
 		capConference = 0x4000,
 		capOffline = 0x8000};
 
@@ -2654,6 +2998,27 @@ public:
 	 */
 	Driver();
 
+	/**
+	 * Set extension numbering scheme.
+	 */
+	virtual bool setExtNumbering(unsigned);
+
+	/**
+	 * Get current extension numbering scheme.
+	 *
+	 * @return extension number size.
+	 */
+	unsigned getExtNumbering(void)
+		{return numCount;};
+
+	/**
+	 * Lookup the extension index entry.
+	 *
+	 * @param extension number.
+	 * @return NULL or index entry reference.
+	 */
+	virtual Trunk **getExtIndex(const char *num);
+
 	/**
 	 * Report if driver is idle.
 	 */
@@ -2698,7 +3063,7 @@ public:
 	 *
 	 * @return trunk timeslots/ports allocated.
 	 */
-	virtual int getTrunkCount(void) = 0;
+	virtual unsigned getTrunkCount(void) = 0;
 
 	/**
 	 * Get active number of ports actually deployed by the current
@@ -2706,7 +3071,7 @@ public:
 	 *
 	 * @return actual trunks used.
 	 */
-	virtual int getTrunkUsed(void) 
+	virtual unsigned getTrunkUsed(void) 
 		{return getTrunkCount();};
 
 	/**
@@ -2729,6 +3094,9 @@ public:
 	TrunkGroup *getCardGroup(int id)
 		{return cards[id];};
 
+	TrunkGroup *getStaGroup(int id)
+		{return stacards[id];};
+
 	/**
 	 * Get the trunk group member trunk.
 	 *
@@ -2748,10 +3116,84 @@ public:
 	/**
 	 * Get a trunk by a id or global id string.
 	 *
+	 * @param trunk or port reference id.
 	 * @return trunk port object.
 	 */
 	Trunk *getTrunkId(const char *id);
 
+	/**
+	 * Get a trunk by extension reference.
+	 *
+	 * @param extension number.
+	 * @return trunk port object.
+	 */
+	Trunk *getExtNumber(const char *ext);
+
+	/**
+	 * Set a physical port to an extension number in the dialing plan.
+	 *
+	 * @param physical extension port.
+	 * @param extension number to associate.
+	 */
+	bool setExtNumber(unsigned id, const char *ext);
+
+	/**
+	 * Set or assign a virtual extension number in the dialing plan.
+	 *
+	 * @param physical extension port tp associate with.
+	 * @param virtual extension number.
+	 */
+	bool setExtVirtual(unsigned id, const char *ext);
+
+	/**
+	 * Clear an extension number entry.
+	 *
+	 * @param extension to clear.
+	 */
+	bool clrExtNumber(const char *ext);
+
+	/**
+	 * Get a trunk by trunk reference.
+	 *
+	 * @param trunk number
+	 * @return trunk port object.
+	 */
+	virtual Trunk *getTrkNumber(const char *trk)
+		{return getTrunkId(trk);};
+
+	/**
+	 * Get a trunk by tie trunk reference.
+	 *
+	 * @param tie trunk id
+	 * @return trunk port object.
+	 */
+	virtual Trunk *getTieNumber(const char *tie)
+		{return NULL;};
+
+	/**
+	 * Get the extension count.
+	 *
+	 * @return total extension count.
+	 */
+	unsigned getExtCount(void)
+		{return extCount;};
+
+	/**
+	 * Get the total trunk count.	
+	 *
+	 * @return trunk count.
+	 */
+	unsigned getTrkCount(void)
+		{return trkCount;};
+
+	/**
+	 * Get thr total tie line count.
+	 *
+	 * @return tie count.
+	 */
+	unsigned getTieCount(void)
+		{return tieCount;};
+	
 	/**
 	 * Get an individual conference resource.
 	 *
@@ -2909,6 +3351,12 @@ public:
 	 */
 	virtual bool debugFifo(char **argv)
 		{return true;};
+
+	/**
+	 * Debug interface for login/logout changes.
+	 */
+	virtual void debugLogin(Trunk *trunk)
+		{return;};
 };
 
 /**
@@ -3130,17 +3578,23 @@ protected:
 class phTone
 {
 private:
+	friend class KeyTones;
 	friend phTone *getphTone(const char *name);
 	static phTone *first;
 	static int ulaw[256];
+	static unsigned char alaw[256];
 	phTone *next;
+
+protected:
 	char name[33];
 	unsigned char *samples;
-	timeout_t duration;
+	timeout_t duration, playtime;
 	unsigned freq1, freq2;
 
 public:
 	static unsigned char linear2ulaw(int sample);
+	static inline unsigned char linear2alaw(int sample)
+		{return alaw[linear2ulaw(sample)];};
 
 	/**
 	 * Create single frequency tone.
@@ -3185,6 +3639,14 @@ public:
 	 */
 	inline timeout_t getDuration(void)
 		{return duration;};
+
+        /**
+         * Get the playtime of the tone.
+         *
+         * @return duration of tone.
+         */
+        inline timeout_t getPlaytime(void)
+                {return playtime;};
 };
 
 /**
@@ -3240,7 +3702,26 @@ public:
 	friend TGI *getInterp(char *cmd);
 };
 
-extern Module *tts, *asr;
+/**
+ * The tts syth classes intercept core functions as needed to create
+ * an interface to a native text to speech subsystem, usually by
+ * manipulating data.play.
+ */
+class TTS
+{
+protected:
+	friend class Plugins;
+
+	TTS();
+
+public:
+	virtual const char *getName(void) = 0;
+
+	virtual const char *getVoices(void)
+		{return "none";};
+
+	virtual bool synth(unsigned id, trunkdata_t *data) = 0;
+};
 
 /**
  * Modules are used for protocol modules and detached threads used to
@@ -3267,6 +3748,8 @@ private:
 	Module *modNext, *sesNext, *cmdNext, *urlNext, *reqNext, *netNext, *symNext;
 
 protected:
+	static Module *modImport;
+
 	Module();
 
 	inline Module *getFirst(void)
@@ -3275,6 +3758,14 @@ protected:
 	inline bool executePrior(Trunk *trunk)
 		{return trunk->execute(prior);};
 
+	/**
+	 * Preprocessor import directives...
+	 *
+	 * @param token
+	 */
+	virtual void import(const char *token)
+		{return;};
+
 	/**
 	 * Get alternate script extensions.
 	 *
@@ -3327,21 +3818,15 @@ protected:
 	 * Add request handler back-end.
 	 */
 	void addRequest(void);
+
+	/**
+	 * Set service thread in trunk object.
+	 *
+	 * @param trunk
+	 * @param service object
+	 */
+	void setThread(Trunk *trunk, Service *svc);
 	
-	/**
-	 * Enable a TTS module.
-	 */
-	inline void enableTTS(void)
-		{tts = this;};
-
-	/**
-	 * Enable a ASR module.
-	 */
-	inline void enableASR(void)
-		{asr = this;};
-
-
-
 	/**
 	 * Parse a url/prompt file into a fixed file path.
 	 */
@@ -3415,6 +3900,12 @@ public:
 	virtual char *dispatch(Trunk *trunk)
 		{return NULL;};
 
+	/**
+	 * Fetch the data pointer of a trunk.
+	 */
+	inline trunkdata_t *getData(Trunk *trunk)
+		{return &trunk->data;};
+
 	/**
 	 * Post execute commit for modules that schedule delay.
 	 *
@@ -3463,29 +3954,6 @@ public:
 		{return NULL;};
 #endif
 
-	/**
-	 * Provide audio output sample from phrase using pure TTS.
-	 *
-	 * @return audio sample object to play.
-	 * @param Trunk reference.
-	 * @param text phrase.
-	 */
-	virtual AudioSample *getPhrase(Trunk *trunk, const char *phrase)
-		{return NULL;};
-
-	/**
-	 * Provide ASR processing of an audio sample using a domain
-	 * vocabulary.
-	 *
-	 * @return text output.
-	 * @param Trunk reference.
-	 * @param Domain.
-	 * @param Digit mask.
-	 * @param Audio sample.
-	 */
-	virtual const char *getSpeech(Trunk *trunk, char **domain, unsigned long mask, AudioSample *sample)
-		{return NULL;};
-
 	/**
 	 * reload provisioning interface for fifos.
 	 */
@@ -3541,29 +4009,6 @@ protected:
 	virtual time_t getExpires(void) = 0;
 };
 
-/**
- * TCP server interface for posting tcp request monitors into the server
- * main watch loop.
- *
- * @short TCP server interface.
- * @author David Sugar <dyfet@ostel.com>
- */
-class TCPServer : public TCPSocket
-{
-private:
-	static unsigned count;
-	static TCPServer *first;
-	TCPServer *next;	
-
-protected:
-	TCPServer(InetAddress &bind, tpport_t port, int backlog = 5);
-
-public:
-	static unsigned setPoll(struct pollfd *pfd, TCPServer **svr);
-
-	virtual void Connect(void) = 0;
-};
-
 /**
  * Protocols are used for processing transactional requests such as
  * those performed thru modules, but thru a TCP protocol.  These support
@@ -3611,11 +4056,52 @@ public:
 		{return sessions;};
 };
 
+/**
+ * The transaction logger can cache transaction results for a given
+ * trunk so that one can later retrieve these results in a rpc service
+ * that monitors transaction results initiated through ccScript services.
+ * This allows a rpc service to asynchronously request a Bayonne script
+ * to run, and then later find out what happened.
+ *
+ * @short transaction log for rpc services
+ * @author David Sugar <dyfet@ostel.com>
+ */
+class RPCTransaction : protected MemPager, protected ThreadLock
+{
+private:
+	typedef struct _tlog
+	{
+		struct _tlog *next;
+		time_t	expires;	// expire transaction
+		bool running;		// exiting flag
+		char id[16];		// trunk seq id
+		char info[32];		// info result
+	}	tlog;
+
+	tlog *flist, *hash[KEYWORD_INDEX_SIZE];
+	unsigned count;
+
+	unsigned getIndex(const char *id);
+
+public:
+	RPCTransaction();
+
+	void expire(void);
+	bool isRunning(const char *id);
+	char *getInfo(const char *id, char *buffer);
+	void setInfo(const char *id, const char *info);
+	void stop(const char *id);
+	const char *start(const char *pid, char **args, time_t timeout, const char *login = "none");
+};
+
+void control(const char *ctrlstring);
 void loader(const char *path, const char *ext);
 statnode_t *getNodes(const char *name);
-timeout_t getSecTimeout(const char *string);
-timeout_t getMSTimeout(const char *string);
+bool isUser(const char *id);
+bool isFHS(void);
+bool hasTTS(void);
 
+extern unsigned numbering;
 extern bool aliases;
 extern bool running;
 extern Keydata application;
@@ -3624,22 +4110,24 @@ extern KeyThreads keythreads;
 extern KeyMemory keymemory;
 extern KeyPaths keypaths;
 extern KeyImports keyimports;
+extern KeyVoices keyvoices;
 extern KeyLocal keylocal;
 extern KeyNetwork keynetwork;
 #ifdef	XML_SCRIPTS
 extern KeyProxy keyproxy;
 #endif
-extern KeyTones keytones;
 extern KeyHandlers keyhandlers;
 extern Plugins plugins;
 extern Driver *driver;
+extern TTS *tts;
 extern Debug *debug;
 extern Monitor *monitor;
-extern Module *asr, *tts;
 extern Fifo fifo;
-#endif
+extern ThreadLock cachelock;
+extern RPCTransaction rpclog;
 
 #ifdef	CCXX_NAMESPACES
 };
 #endif
 
+#endif
