@@ -14,14 +14,23 @@
 // along with this program; if not, write to the Free Software 
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
+#ifdef  __FreeBSD__
+#define getopt(a, b, c) getopt()
+#include <unistd.h>
+#undef getopt
+#endif
+
 #include <cc++/config.h>
 #include <cc++/process.h>
 #include <cc++/url.h>
 #include <getopt.h>
 #include <sys/wait.h>
 #include <sys/utsname.h>
+#include <netinet/tcp.h>
+#include <sys/un.h>
 #include <cerrno>
 #include "server.h"
+#include <iomanip>
 
 #ifdef	DLL_SERVER
 #define	PLUGIN_SUBDIR	".libs/"
@@ -38,7 +47,9 @@ namespace ost {
 using namespace std;
 #endif
 
+ThreadLock cachelock;
 bool running = false;
+unsigned numbering = 0;
 int mainpid;
 int tgipipe[2];
 bool restart_server = false;
@@ -48,6 +59,70 @@ static int *tgipids = NULL;
 static Resolver *resolver = NULL;
 static char **restart_argv;
 static char **cmds;
+static char *inits[128];
+static unsigned icount = 0;
+
+static int pidfile(char *fpath)
+{
+	int fd;
+	pid_t pid;
+	time_t now;
+	struct stat ino;
+	int len, status;
+	int mask;
+	char buffer[65];
+
+	for(;;)
+	{
+		fd = open(fpath, O_WRONLY | O_CREAT | O_EXCL, 0660);
+		if(fd > 0)
+		{
+			pid = getpid();
+			snprintf(buffer, sizeof(buffer), "%d\n", pid);
+			write(fd, buffer, strlen(buffer));
+			close(fd);
+			return 0;
+		}
+		if(fd < 0 && errno != EEXIST)
+			return -1;
+
+		fd = open(fpath, O_RDONLY);
+		if(fd < 0)
+		{
+			if(errno == ENOENT)
+				continue;
+			return -1;
+		}
+		sleep(2);
+		status = read(fd, buffer, sizeof(buffer) - 1);
+		if(status < 1)
+		{
+			close(fd);
+			continue;
+		}
+
+		buffer[status] = 0;
+		pid = atoi(buffer);
+		if(pid)
+		{
+			if(pid == getpid())
+			{
+				status = -1;
+				errno = 0;
+			}
+			else
+				status = kill(pid, 0);
+
+			if(!status || (errno == EPERM))
+			{
+				close(fd);
+				return pid;
+			}
+		}
+		close(fd);
+		unlink(fpath);
+	}
+}
 
 #ifndef	HAVE_SETENV
 static void setenv(char *name, const char *value, int overwrite)
@@ -61,27 +136,43 @@ static void setenv(char *name, const char *value, int overwrite)
 
 static void purge(const char *path)
 {
+	if(!isDir(path))
+		return;
+
+	char fullpath[256];
 	char *ex[4];
 	int pid;
+	Dir dir(path);
+	char *name;
 
-	pid = fork();
-	if(pid < 0)
-		return;
-
-	if(pid)
+	while(NULL != (name = dir.getName()))
 	{
-		waitpid(pid, NULL, 0);
-		return;
+		if(!stricmp(name, "."))
+			continue;
+
+		if(!stricmp(name, ".."))
+			continue;
+
+		pid = fork();
+		if(pid < 0)
+			return;
+
+		if(pid)
+		{
+			waitpid(pid, NULL, 0);
+			continue;
+		}
+
+		snprintf(fullpath, sizeof(fullpath), "%s/%s", path, name);
+		ex[0] = "rm";
+		ex[1] = "-rf";
+		ex[2] = fullpath;
+		ex[3] = NULL;
+
+		exit(execvp("rm", ex));
 	}
-
-	ex[0] = "rm";
-	ex[1] = "-rf";
-	ex[2] = (char *)path;
-	ex[3] = NULL;
-
-	exit(execvp("rm", ex));
 }
-
+		
 static void rt(void)
 {
 	int min, max;
@@ -89,12 +180,12 @@ static void rt(void)
 	int pri = keythreads.getPriority();
 	int policy = keythreads.getPolicy();
 	char memory[1024 * pages];
-	
-	memset(memory, 0, sizeof(memory));
+
+	memset(memory, sizeof(memory), 0);
 
 #ifdef	_POSIX_PRIORITY_SCHEDULING
 	struct sched_param p;
-	
+
 	sched_getparam(0, &p);
 	pri += p.sched_priority;
 	if(!policy)
@@ -107,7 +198,7 @@ static void rt(void)
 	if(pri > max)
 		pri = max;
 	p.sched_priority = pri;
-	sched_setscheduler(getpid(), policy, &p); 
+	sched_setscheduler(getpid(), policy, &p);
 #else
 	nice(-pri);
 #endif
@@ -118,6 +209,8 @@ static void rt(void)
 #endif
 }
 
+#ifdef	HAVE_TGI
+
 static RETSIGTYPE tgiusr1(int signo)
 {
 }
@@ -127,7 +220,7 @@ static void tgi(int cfd)
 	int argc;
 	char *argv[65];
 	char *user;
-char buffer[PIPE_BUF];
+	char buffer[PIPE_BUF / 2];
 	int vpid, pool, stat;
 	int count = keythreads.getGateways();
 	tgipids = new int[count];
@@ -481,6 +574,7 @@ char buffer[PIPE_BUF];
 	exit(SIGINT);
 }
 
+/*
 static void tgisignal(void)
 {
 	int count = keythreads.getGateways();
@@ -492,11 +586,15 @@ static void tgisignal(void)
 	for(pid = 0; pid < count; ++pid)
 		kill(tgipids[pid], SIGUSR1);
 }
+*/
+
+#endif
 		
 static RETSIGTYPE final(int sig)
 {
 	int count = keythreads.getGateways();
 	int i;
+	const char *cp;
 
 	if(debug)
 		if(debug->debugFinal(sig))
@@ -525,6 +623,8 @@ static RETSIGTYPE final(int sig)
 	if(resolver)
 		delete resolver;
 
+	Trunk::sync();	
+
 	if(driver)
 		driver->stop();
 
@@ -537,14 +637,33 @@ static RETSIGTYPE final(int sig)
 	{
 		for(i = 0; i < count; ++i)
 		{
-			if(!kill(tgipids[i], SIGINT))
-				waitpid(tgipids[i], NULL, 0);
+			kill(tgipids[i], SIGINT);
+			waitpid(tgipids[i], NULL, 0);
 		}
 		tgipids = NULL;
 	}
 
 	if(restart_server)
 		execvp(*restart_argv, restart_argv);
+
+	cp = keypaths.getLast("pidfile");
+	if(cp)
+		remove(cp);
+
+	cp = keypaths.getLast("ctrlfile");
+	if(cp)
+		remove(cp);
+
+        cp = keypaths.getLast("pktfile");
+        if(cp)
+                remove(cp);
+
+	cp = keypaths.getLast("nodefile");
+	if(cp)
+		remove(cp);
+
+	purge(keypaths.getLast("tmpfs"));
+	purge(keypaths.getLast("tmp"));
 	exit(sig);
 }
 
@@ -566,13 +685,15 @@ static void initial(int argc, char **argv, int cfd)
 		{"voice", 1, 0, 'v'},
 		{"language", 1, 0, 'l'},
 		{"gui", 0, 0, 'G'},
-		{"alt", 1, 0, 'a'},
 		{"display", 1, 0, 'X'},
-		{"secure", 0, 0, 'S'},
+		{"startup", 1, 0, 'S'},
 		{"debug", 0, 0, 'x'},
-		{"start", 1, 0, 's'},
 		{"demo", 0, 0, 'c'},
+		{"init", 0, 0, 'I'},
+		{"alt", 1, 0, 'a'},
 		{"groups", 0, 0, 'g'},
+		{"tts", 1, 0, 'Y'},
+		{"prefix", 1, 0, 'z'},
 		{0, 0, 0, 0}};
 
 	static bool daemon = false;
@@ -580,7 +701,6 @@ static void initial(int argc, char **argv, int cfd)
 	static bool gui = false;
 	static bool pcount = 0;
 	static bool mcount = 0;
-	static bool su = false;
 	static unsigned plen = 0;
 
 	TrunkGroup *grp;
@@ -591,21 +711,21 @@ static void initial(int argc, char **argv, int cfd)
 	char *envDriver = getenv("BAYONNE_DRIVER");
 	Mixer *mixer;
 	fstream mix;
-	char prefix[256];
+	char prefix[256], libpath[256];
 	const char *cp;
 	char *pp, *tok;
 	sigset_t sigs;
 	int opt, opt_index;
-	char *opt_script = NULL;
-	int ports, id;
+	unsigned ports, id;
 	bool test = false;
 	bool schedule = true;
-	struct stat ino;
-	int count, spid = 0;
+	unsigned count;
 	ofstream php;
-	const char *embed;
+	const char *embed, *name;
 	bool gdump = false;
+	bool pref = false;
 	const char *alt = NULL;
+	char *sp = NULL;
 
 	cmds = argv;
 	mainpid = getpid();
@@ -620,31 +740,42 @@ static void initial(int argc, char **argv, int cfd)
 	pp = strrchr(prefix, '/');
         if(pp)
         {
-                *pp = 0;
+        	*pp = 0;
                 chdir(prefix);
         }
-	keypaths.setPrefix(prefix);
 
 	uname(&uts);
 
 	getcwd(prefix, sizeof(prefix) - 1);
 	pp = strrchr(prefix, '/');
 
+	snprintf(libpath, sizeof(libpath), "%s/%s", 
+		keypaths.getLast("libpath"), VERPATH);
+	keypaths.setValue("modlibpath", libpath);
+
 	if(envDriver)
 		plugins.setValue("driver", envDriver);
 
-	while(EOF != (opt = getopt_long(argc, argv, "xP:GTDFX:hd:p:d:tn:a:ASs:v:l:g", long_options, &opt_index)))
+	while(EOF != (opt = getopt_long(argc, argv, "z:123cthVxP:GTDFX:d:p:d:n:a:AS:s:v:l:gY:I", long_options, &opt_index)))
 		switch(opt)
 		{
-		case 's':
-			// printf("set %s\n", optarg);
-			spid = atoi(optarg);
+		case 'z':
+			sp = optarg;
+			break;
+		case '1':
+			numbering = 1;
+			break;
+		case '2':
+			numbering = 2;
+			break;
+		case '3':
+			numbering = 3;
 			break;
 		case 'x':
 			slog.level(Slog::levelDebug);
 			break;
-		case 'S':
-			su = true;
+		case 'I':
+			pref = true;
 			break;
 		case 'g':
 			gdump = true;
@@ -652,6 +783,9 @@ static void initial(int argc, char **argv, int cfd)
 		case 'v':
 			setenv("BAYONNE_VOICE", optarg, 1);
 			break;
+		case 'Y':
+			plugins.setValue("tts", optarg);
+			break;
 		case 'l':
 			setenv("BAYONNE_LANGUAGE", optarg, 1);
 			break;
@@ -665,6 +799,9 @@ static void initial(int argc, char **argv, int cfd)
 			plugins.setValue("debug", "tcpmon");
 			setenv("TCPMON", optarg, 1);
 			break;	
+		case 'S':
+			inits[icount++] = optarg;
+			break;
 		case 'V':
 			cout << VERPATH << endl;
 			exit(0);
@@ -678,7 +815,7 @@ static void initial(int argc, char **argv, int cfd)
 			plugins.setValue("debug", "trace");
 			break;
 		case 'a':
-			alt = optarg;			
+			alt = optarg;
 		case 't':
 			schedule = false;
 			test = true;
@@ -704,9 +841,21 @@ static void initial(int argc, char **argv, int cfd)
 			strcpy(pp, "/modules/translators/" PLUGIN_SUBDIR "english.tts");
 			plugins.setValue("languages", prefix);
 			plugins.setValue("switch", "");
+#ifdef	HAVE_FLITE
+			strcpy(pp, "/modules/flite/" PLUGIN_SUBDIR "flite.tts");
+			plugins.setValue("tts", prefix);
+#endif
+
+#ifdef	HAVE_POSTGRES
+			strcpy(pp, "/modules/postgres/" PLUGIN_SUBDIR "postgres.sql");
+			plugins.setValue("sql", prefix);
+#endif
+
 #ifdef	COMMON_XML_PARSING
 			strcpy(pp, "/modules/xml/" PLUGIN_SUBDIR "bayonne.xml");
 			plugins.setValue("xml", prefix);
+			strcpy(pp, "/modules/xml" PLUGIN_SUBDIR);
+			keypaths.setValue("modlibpath", prefix);
 #endif
 //			strcpy(pp, "/modules/perl/" PLUGIN_SUBDIR "perl.tgi");
 //			plugins.setValue("tgi", prefix);
@@ -718,25 +867,25 @@ static void initial(int argc, char **argv, int cfd)
 			else
 				strcpy(pp, "/server/" PLUGIN_SUBDIR "test.dbg");
 			plugins.setValue("debug", prefix);
+			keypaths.setValue("cache", "cache");
+			keypaths.setValue("spool", "spool");
 			strcpy(pp, "/data/script");
 			keypaths.setValue("scripts", prefix);
 			strcpy(pp, "/data");
 			keypaths.setValue("prompts", prefix);
+			if(alt)
+				cp = strrchr(alt, '/');
+			else
+				cp = "/alt";
+			if(!cp)
+				cp = alt;
+			else
+				++cp;
+			sprintf(pp, "/data/%s", cp);
 			keypaths.setValue("libexec", prefix);
-
-                        keypaths.setValue("prompts", prefix);
-                        if(alt)
-                                cp = strrchr(alt, '/');
-                        else
-                                cp = "/alt";
-                        if(!cp)
-                                cp = alt;
-                        else
-                                ++cp;
-                        sprintf(pp, "/data/%s", cp);
 			keypaths.setValue("altprompts", prefix);
 			keypaths.setValue("altscripts", prefix);
-                        keypaths.setValue("datafiles", "../var");
+			keypaths.setValue("datafiles", "../var");
 			break;
 		case 'n':
 			keyserver.setValue("node", optarg);
@@ -788,10 +937,9 @@ static void initial(int argc, char **argv, int cfd)
 	}
 
 	endKeydata();
+	if(sp)
+		keypaths.setPrefix(sp);
 
-	if(spid != getpid() && !test)
-		clog << "*** please use \"bayonne_start\" in a production environment" << endl;
-	
 	keyserver.setGid();
 
 	if(!getuid())
@@ -803,20 +951,19 @@ static void initial(int argc, char **argv, int cfd)
 	mkdir(keypaths.getDatafiles(), 0750);
 	chdir(keypaths.getDatafiles());
 	purge("temp");
-	purge("cache");
+	purge(keypaths.getSpool());
+	purge(keypaths.getCache());
 	purge(keypaths.getLast("tmpfs"));
 	purge(keypaths.getLast("tmp"));
 
 	mkdir(keypaths.getLast("tmpfs"), 0770);
 	mkdir(keypaths.getLast("tmp"), 0770);
 	mkdir("temp", 0770);
-	mkdir("cache", 0770);
-
-	for(count = 0; count < 100; ++count)
-	{
-		sprintf(prefix, "cache/%02d", count);
-		mkdir(prefix, 0770);
-	}
+	mkdir(keypaths.getCache(), 0770);
+	mkdir("users", 0770);
+	symlink(keypaths.getCache(), "cache");
+	mkdir(keypaths.getSpool(), 0770);
+	symlink(keypaths.getSpool(), "spool");
 
 	if(daemon)
 	{
@@ -843,9 +990,28 @@ static void initial(int argc, char **argv, int cfd)
 		slog(Slog::levelNotice) << "daemon init started" << endl;
 	}
 
-	sprintf(prefix, "%s/bayonne", keypaths.getLibexec());
+	if(canModify(keypaths.getRunfiles()))
+		snprintf(prefix, sizeof(prefix), "%s/bayonne.pid",
+			keypaths.getRunfiles());
+	else
+		snprintf(prefix, sizeof(prefix), "%s/.bayonne.pid",
+			Process::getEnv("HOME"));
+
+	keypaths.setValue("pidfile", prefix);
+
+	switch(pidfile(prefix))
+	{
+	case -1:
+		slog(Slog::levelWarning) << "server: cannot create pidfile " << prefix << endl;
+	case 0:
+		break;
+	default:
+		slog(Slog::levelCritical) << "server: another instance running; cannot continue" << endl;
+		final(-1);
+	}
+
 	setenv("SERVER_PLATFORM", plugins.getDriverName(), 1);
-	setenv("SERVER_LIBEXEC", prefix, 1);
+	setenv("SERVER_LIBEXEC", keypaths.getLibexec(), 1);
 	setenv("SERVER_SOFTWARE", "bayonne", 1);
 	setenv("SERVER_PROTOCOL", "2.2", 1);
 	setenv("SERVER_VERSION", VERPATH, 1);
@@ -880,12 +1046,35 @@ static void initial(int argc, char **argv, int cfd)
 		slog(Slog::levelCritical) << dso->getError() << endl;
 		final(-1);
 	}
+#ifdef	HAVE_TGI
 	tgi(cfd);
 	sleep(1);
+#endif
 	rt();
 
-	if(su)
-		keyserver.setUid();
+#ifdef	SCRIPT_IF_OVERRIDE
+	addConditional("voice", &Trunk::hasVoice);
+	addConditional("alt", &Trunk::hasAltVoice);
+	addConditional("altvoice", &Trunk::hasAltVoice);
+	addConditional("sys", &Trunk::hasSysVoice);
+	addConditional("sysvoice", &Trunk::hasSysVoice);
+	addConditional("app", &Trunk::hasAppVoice);
+	addConditional("appvoice", &Trunk::hasAppVoice);
+	addConditional("group", &Trunk::hasGroup);
+	addConditional("policy", &Trunk::hasGroup);
+	addConditional("plugin", &Trunk::hasPlugin);
+	addConditional("node", &Trunk::isNode);
+	addConditional("service", &Trunk::isService);
+	addConditional("dtmf", &Trunk::ifDTMF);
+	addConditional("feature", &Trunk::ifFeature);
+	addConditional("ext", &Trunk::isExtension);
+	addConditional("extension", &Trunk::isExtension);
+	addConditional("station", &Trunk::isStation);
+	addConditional("virtual", &Trunk::isVirtual);
+	addConditional("user", &Trunk::isActiveUser);
+	addConditional("dnd", &Trunk::isDnd);
+	addConditional("hunt", &Trunk::isHunt);
+#endif
 
 	try
 	{
@@ -897,6 +1086,8 @@ static void initial(int argc, char **argv, int cfd)
 		plugins.loadMonitor();
 		plugins.loadDriver();
 		plugins.loadSwitch();
+		plugins.loadTTS();
+		plugins.loadSQL();
 		plugins.loadModules();
 		plugins.loadPreload();
 		plugins.loadTranslators();
@@ -904,7 +1095,10 @@ static void initial(int argc, char **argv, int cfd)
 #ifdef	XML_SCRIPTS
 		plugins.loadXML();
 #endif
-		driver->getImage();
+		if(!(driver->getCaps() & Driver::capSwitch))
+			driver->getImage();
+		else if(numbering)
+			driver->setExtNumbering(numbering);
 	}
 	catch(Driver *drv)
 	{
@@ -1136,6 +1330,47 @@ static void initial(int argc, char **argv, int cfd)
 	globals->setConst(SYM_PROMPTS, keypaths.getPromptFiles());
 	globals->setConst(SYM_RELEASE, "1");
 
+	Trunk::initPassword();
+	Trunk::initLines();
+
+	Dir users("users");
+
+	while(NULL != (name = users.getName()))
+	{
+		if(*name == '.')
+			continue;
+
+		Trunk::load(name);
+	}
+
+	if(isDir("lines"))
+	{
+		Dir lines("lines");
+	
+		while(NULL != (name = lines.getName()))
+		{
+			if(*name == '.')
+				continue;
+
+			Trunk::loadPref(name, "line", "lines");
+		}
+	}
+
+	if(isDir("hunting"))
+	{
+		Dir hunt("hunting");
+
+		while(NULL != (name = hunt.getName()))
+		{
+			if(*name == '.')
+				continue;
+
+			Trunk::loadPref(name, "hunt", "hunting");
+		}
+	}
+
+	Trunk::sync(pref);
+	new KeyTones();
 	running = true;
 }
 
@@ -1160,29 +1395,160 @@ void loader(const char *path, const char *ext)
 	}
 }
 
+void check(void)
+{
+	Dir dir(keypaths.getCache());
+	const char *entry;
+	char path[128];
+	time_t now;
+	struct stat ino;
+
+	slog(Slog::levelDebug) << "server: checking cache..." << endl;
+
+	time(&now);
+
+	while(NULL != (entry = dir.getName()))
+	{
+		if(*entry == '.')
+			continue;
+
+		snprintf(path, sizeof(path), "cache/%s", entry);
+		if(stat(path, &ino))
+			continue;
+
+		if(now - ino.st_mtime >= 3600)
+		{
+			cachelock.writeLock();
+			remove(path);
+			cachelock.unlock();
+		}
+	}
+}
+
+static int fd = -1;
+static int so = -1;
+
+void control(const char *cmd)
+{
+	if(fd < 0)
+		return;
+
+	write(fd, cmd, strlen(cmd));
+}
+
+#define	PROTOCOL_VERSION 2
+
+static void packetio(void)
+{
+        bool rts;
+        char packet[512];
+        char *p = packet + 1;
+        char reply[2];
+        struct sockaddr_un caddr;
+        socklen_t clen = sizeof(caddr);
+        int len = ::recvfrom(so, packet, sizeof(packet), 0, (struct sockaddr *)&caddr, &clen);
+
+        if(len < 0)
+        {
+                slog(Slog::levelWarning) << "packet: invalid read" << endl;
+                return;
+        }
+
+        while(isspace(*p))
+                ++p;
+        rts = fifo.command(p);
+        if(!packet[0])
+                return;
+
+        reply[0] = packet[0];
+        if(rts)
+                reply[1] = '1';
+        else
+                reply[0] = '0';
+
+        ::sendto(so, reply, 2, 0, (struct sockaddr *)&caddr, len);
+}
+
 extern "C" int main(int argc, char **argv)
 {
-	static char buffer[PIPE_BUF];
+	static char buffer[256];
+	static char packet[256];
+	struct sockaddr_un server_addr;
 	char *p;
-	int bpos = 0;
-	int i, fd;
-	struct pollfd pfd[33];
+	const char *etc;
+	int bpos = 0, len;
+#ifdef	HAVE_POLL
+	struct pollfd pfd[2];
+#else
+	struct timeval tv;
+	int maxfd;
+	fd_set sfd;
+#endif
 	TimerPort timer;
 	timeout_t step;
-	TCPServer *tcp[32];
-	int count;
-	unsigned seccount = 0;
+	unsigned ic = 0;
+	unsigned seccount = 0, mincount = 0, minhour = 0;
 	ifstream init;
+#ifndef NODE_SERVICES
+        statnode_t node;
+        static char nodepath[256];
+        int nodes;
+        TrunkGroup *grp;
+#endif
 
-	if(canModify(keypaths.getRunfiles()))
-		snprintf(buffer, sizeof(buffer), "%s/bayonne.ctrl",
-			keypaths.getRunfiles());
-	else
-		snprintf(buffer, sizeof(buffer), "%s/.bayonne.ctrl",
-			getenv("HOME"));
+        if(canModify(keypaths.getRunfiles()))
+        {
+                snprintf(buffer, sizeof(buffer), "%s/bayonne.ctrl",
+                        keypaths.getRunfiles());
+                snprintf(packet, sizeof(packet), "%s/bayonne.pkt",
+                        keypaths.getRunfiles());
+
+#ifndef NODE_SERVICES
+                snprintf(nodepath, sizeof(nodepath), "%s/bayonne.nodes",
+                        keypaths.getRunfiles());
+#endif
+        }
+        else
+        {
+                snprintf(buffer, sizeof(buffer), "%s/.bayonne.ctrl",
+                        getenv("HOME"));
+                snprintf(packet, sizeof(packet), "%s/.bayonne.pkt",
+                        getenv("HOME"));
+
+#ifndef NODE_SERVICES
+                snprintf(nodepath, sizeof(nodepath), "%s/.bayonne.nodes",
+                        keypaths.getRunfiles());
+#endif
+        }
+
+	keypaths.setValue("ctrlfile", buffer);
+	keypaths.setValue("pktfile", packet);
 
 	if(!getuid())
 		umask(003);
+        memset(&server_addr, 0, sizeof(server_addr));
+        server_addr.sun_family = AF_UNIX;
+        strncpy(server_addr.sun_path, packet, sizeof(server_addr.sun_path));
+
+#ifdef  __SUN_LEN
+        len = sizeof(server_addr.sun_len) + strlen(server_addr.sun_path)
+                        + sizeof(addr.sun_family) + 1;
+
+        addr.sun_len = len;
+#else
+        len = strlen(server_addr.sun_path) + sizeof(server_addr.sun_family) + 1;
+#endif
+        remove(packet);
+        so = socket(AF_UNIX, SOCK_DGRAM, 0);
+        if(so > -1)
+        {
+                if(bind(so, (struct sockaddr *)&server_addr, len))
+                {
+                        slog(Slog::levelError) << "packet interface failed;" << endl;
+                        so = -1;
+                        remove(packet);
+                }
+        }
 
 	remove(buffer);
 	mkfifo(buffer, 0660);
@@ -1190,8 +1556,6 @@ extern "C" int main(int argc, char **argv)
 
 	initial(argc, argv, fd);
 
-	chown(buffer, keyserver.getUid(), keyserver.getGid());
-
 	if(debug->debugTest())
 		final(0);
 
@@ -1202,9 +1566,36 @@ extern "C" int main(int argc, char **argv)
 		sleep((unsigned)(-1));
 	}
 
+	chown(buffer, keyserver.getUid(), keyserver.getGid());
+#ifndef NODE_SERVICES
+        nodes = creat(nodepath, 0640);
+        chown(nodepath, keyserver.getUid(), keyserver.getGid());
+        grp = getGroup();
+#endif
+
 	scheduler.initial();
 
-	snprintf(buffer, sizeof(buffer), "%sbayonne.init", keypaths.getLast("etc"));
+	while(ic < icount)
+		fifo.command(inits[ic++]);
+		
+        etc = keypaths.getLast("etc");
+        etc = strchr(etc + 1, '/');
+        if(etc)
+                ++etc;
+        else
+                etc = "";
+
+        strcpy(buffer, keypaths.getLast("etc"));
+        if(*etc)
+                strcat(buffer, "startup.conf");
+        else
+                strcat(buffer, "bayonne.init");
+        if(!canAccess(buffer))
+        {
+                strcpy(buffer, keypaths.getLast("etc"));
+                strcat(buffer, "bayonne.init");
+        }
+
 	init.open(buffer);
 	if(init.is_open())
 	{
@@ -1232,35 +1623,85 @@ extern "C" int main(int argc, char **argv)
 	for(;;)
 	{
 		Thread::yield();
+#ifdef	HAVE_POLL
 		pfd[0].fd = fd;
 		pfd[0].events = POLLIN | POLLRDNORM;
 		pfd[0].revents = 0;
-
-		count = TCPServer::setPoll(&pfd[1], tcp) + 1;
-
+		if(so > -1)
+		{
+	                pfd[1].fd = so;
+        	        pfd[1].events = POLLIN | POLLRDNORM;
+                	pfd[1].revents = 0;
+		}
+#else
+		maxfd = fd;
+		FD_ZERO(&sfd);
+		FD_SET(fd, &sfd);
+		if(so > -1)
+			FD_SET(so, &sfd);
+#endif
 		step = timer.getTimer();
 		if(!step)
 		{
 			if(driver)
 				driver->secTick();
 			timer.setTimer(1000);
+#ifndef NODE_SERVICES
+                        if(!(seccount % 10) && nodes > -1)
+                        {
+                                lseek(nodes, 0l, SEEK_SET);
+                                memset(&node, 0, sizeof(node));
+                                node.version = PROTOCOL_VERSION;
+                                node.buddies = 0;
+                                strncpy(node.name, keyserver.getNode(),
+                                        sizeof(node.name));
+                                node.ports = driver->getTrunkCount();
+                                node.calls = grp->getStat(STAT_SYS_ACTIVITY);
+                                driver->getStatus(node.stat);
+				time(&node.update);
+                                ::write(nodes, &node, sizeof(node));
+                        }
+#endif
+
 			if(++seccount >= 60)
 			{
 				scheduler.sync();
+				rpclog.expire();
 				seccount = 0;
-			}
+				if(++mincount >= 10)
+				{
+					check();
+					mincount = 0;
+				}
+				if(++minhour >= 60)
+				{
+					Trunk::sync();
+					minhour = 0;
+				}
+			}	
 			continue;
 		}
-		poll(pfd, count, step);
-
-		while(--count)
-		{
-			if(pfd[count + 1].revents)
-				tcp[count]->Connect();
-		}
 
+#ifdef	HAVE_POLL
+		len = 1;
+		if(so > -1)
+			++len;
+		poll(pfd, len, step);
+		if(so > -1 && pfd[1].revents & POLLIN)
+			packetio();
 		if(pfd[0].revents & POLLIN)
                 {
+#else
+		tv.tv_sec = step / 1000;
+		tv.tv_usec = (step % 1000) * 1000;
+		select(maxfd + 1, &sfd, NULL, &sfd, &tv);
+
+		if(so > -1 && FD_ISSET(so, &sfd))
+			packetio();
+
+		if(FD_ISSET(fd, &sfd))
+		{
+#endif
                         bpos = 0;
                         for(;;)
                         {
@@ -1283,7 +1724,7 @@ extern "C" int main(int argc, char **argv)
 			while(*p == ' ' || *p == '\t')
 				++p;
 			if(!*p)
-			continue;
+				continue;
 			fifo.command(p);
 		}
 	}
