@@ -21,6 +21,14 @@
 namespace ost {
 #endif
 
+#ifdef	HAVE_TGI
+
+#if PIPE_BUF > 1024
+#define	TGI_BUF (1024)
+#else
+#define	TGI_BUF	(512)
+#endif
+
 typedef enum
 {
 	TGI_EXEC_NORMAL,
@@ -33,9 +41,11 @@ typedef	struct
 {
 	unsigned short port;
 	tgiexec_t mode;
-char cmd[PIPE_BUF - sizeof(short) - sizeof(tgiexec_t)];
+	char cmd[TGI_BUF - sizeof(short) - sizeof(tgiexec_t)];
 } tgicmd_t;
 
+#endif
+
 typedef struct {
 	time_t update;
 	statnode_t *buddy;
@@ -86,8 +96,8 @@ private:
 	char *getPath(void);
 	statnode_t *getNode(const char *name);
 	statnode_t *getAddr(struct in_addr *addr);
-	struct sockaddr bcast;
-	struct sockaddr_in *bcast_in;
+	struct sockaddr bcast, maddr;
+	struct sockaddr_in *bcast_in, *maddr_in;
 
 	void initial(void);
 	void run(void);
@@ -157,7 +167,8 @@ public:
 	Resolver();
 };
 
-#define	SYM_NETWORK	"pstn.network"
+#define	SYM_NETWORK	"driver.network"
+#define	SYM_TRUNKID	"driver.trunkid"
 #define	SYM_INTERFACE	"pstn.interface"
 #define	SYM_TONE	"pstn.tone"
 #define	SYM_PROGRESS	"pstn.tone"
@@ -170,6 +181,8 @@ public:
 #define	SYM_OFFSET	"audio.offset"
 #define	SYM_BUFFER	"audio.buffer"
 #define	SYM_LANGUAGE	"session.language"
+#define	SYM_CALLTYPE	"session.calltype"
+#define	SYM_CALLFWD	"session.callfwd"
 #define	SYM_TRIM	"audio.trim"
 #define	SYM_VOICE	"session.voice"
 #define	SYM_VOLUME	"audio.volume"
@@ -194,6 +207,8 @@ public:
 #define	SYM_DURATION	"session.duration"
 #define	SYM_USER	"server.user"
 #define	SYM_ID		"driver.id"
+#define	SYM_CARD	"driver.card"
+#define	SYM_BANK	"driver.bank"
 #define	SYM_SPAN	"driver.span"
 #define	SYM_PORTS	"server.ports"
 #define	SYM_XML		"session.xml"
@@ -208,16 +223,24 @@ public:
 #define	SYM_SCRIPTS	"server.scripts"
 #define	SYM_PROMPTS	"server.prompts"
 #define	SYM_POLICIES	"server.policies"
-#define	SYM_POLICY	"policy.name"
+#define	SYM_EXTNUMBER	"session.extension"
+#define	SYM_POLICY	"session.policyid"
 #define	SYM_START	"session.home"
 #define	SYM_BASE	"session.base"
 #define	SYM_DATE	"session.date"
 #define	SYM_TIME	"session.time"
 #define	SYM_COUNT	"session.count"
 #define	SYM_DIGITS	"session.digits"
+#define	SYM_JOINID	"session.joinid"
+#define	SYM_PICKUP	"session.pickupid"
+#define	SYM_RECALL	"session.recallid"
+#define	SYM_TRUNK	"session.trunkid"
+#define	SYM_TRANSFER	"session.transferid"
+#define	SYM_INTERCOM	"session.intercomid"
 #define SYM_EVENTID     "session.eventsenderid"
 #define SYM_EVENTMSG    "session.eventsendermsg"
 #define	SYM_HOME	"session.home"
+#define	SYM_LOGIN	"session.loginid"
 #define	SYM_PARENT	"session.parent"
 
 #define	SYM_RINGS	"pstn.rings"
