@@ -49,12 +49,16 @@ ScriptInterp((aaScript *)driver, keymemory.getSymbolSize(), keymemory.getPageSiz
 
 	++driver->portCount;
 
+	for(i = 0; i < 10; ++i)
+		dialgroup[i] = false;
+
 	flags.dtmf = flags.offhook = flags.reset = false;
 	flags.trunk = TRUNK_MODE_INACTIVE;
 	flags.dsp = DSP_MODE_INACTIVE;
 	flags.script = false;
 	flags.ready = false;
-	flags.onexit = false;
+	flags.onexit = false;	
+	flags.dnd = false;
 	id = port;
 	span = dspan;
 	rings = 0;
@@ -64,8 +68,10 @@ ScriptInterp((aaScript *)driver, keymemory.getSymbolSize(), keymemory.getPageSiz
 	digits = 0;
 #ifdef	XML_SCRIPTS
 	altimage = NULL;
+	altmodule = NULL;
 #endif
 	apppath[0] = 0;
+	extNumber[0] = 0;
 	seq = 0;
 	tonetmp = NULL;
 
@@ -77,7 +83,6 @@ ScriptInterp((aaScript *)driver, keymemory.getSymbolSize(), keymemory.getPageSiz
 	dtmf.bin.flags.commit = true;
 	dtmf.bin.flags.size = MAX_DIGITS;
 	dtmf.bin.next = NULL;
-	dtmf.bin.data[0] = 0;
 
 	for(i = 0; i < 6; ++i)
 	{
@@ -91,6 +96,28 @@ ScriptInterp((aaScript *)driver, keymemory.getSymbolSize(), keymemory.getPageSiz
 		numbers[i].sym.next = NULL;
 	}
 	strcpy(numbers[5].sym.data, "init");
+	dtmf.bin.data[0] = 0;
+}
+
+bool Trunk::isAdmin(void)
+{
+	const char *cp = getSymbol(SYM_LOGIN);
+	if(!cp)
+		return false;
+
+	if(!stricmp(cp, "admin"))
+		return true;
+
+	return false;
+}
+
+const char *Trunk::getStation(void)
+{
+	const char *kw = getKeyword("station");
+	if(!kw)
+		kw = group->getLast("station");
+
+	return kw;
 }
 
 void Trunk::setIdle(bool mode)
@@ -110,9 +137,9 @@ phTone *Trunk::getTone(void)
 		delete tonetmp;
 
 	if(!data.tone.freq2)
-		tonetmp = new phTone("temp", data.tone.duration, data.tone.freq1);
+		tonetmp = new phTone(NULL, data.tone.duration, data.tone.freq1);
 	else
-		tonetmp = new phTone("temp", data.tone.duration, data.tone.freq1, data.tone.freq2);
+		tonetmp = new phTone(NULL, data.tone.duration, data.tone.freq1, data.tone.freq2);
 
 	data.tone.tone = tonetmp;
 	return tonetmp;
@@ -138,23 +165,22 @@ int Trunk::getDigit(char dig)
 	return -1;
 }
 
+#ifdef	HAVE_TGI
 void Trunk::libtts(const char *msg, ttsmode_t mode)
 {
 	tgicmd_t cmd;
-	int argc = 1;
 #ifdef	HAVE_SSTREAM
 	ostringstream str;
 	str.str() = "";
 #else
 	strstream str(cmd.cmd, sizeof(cmd.cmd));
 #endif
-	char *qc;
 
 	cmd.port = id;
 	cmd.mode = TGI_EXEC_AUDIO;
 	cmd.cmd[0] = 0;
 
-	str << keyhandlers.getLast("tts") << ".tts";
+	str << keyhandlers.getLast("say") << ".tts";
 	str << " language=" << getSymbol(SYM_LANGUAGE);
 	str << " audio=" << "temp/.tts." << id << ".ul";
 	str << " voice=" << getSymbol(SYM_VOICE);
@@ -175,61 +201,7 @@ void Trunk::libtts(const char *msg, ttsmode_t mode)
 	::write(tgipipe[1], &cmd, sizeof(cmd));
 }
 
-void Trunk::libexec(char **argv, unsigned timeout)
-{
-	tgicmd_t cmd;
-	int argc = 1;
-	const char *token = keyserver.getToken();
-#ifdef	HAVE_SSTREAM
-	ostringstream str;
-	str.str() = "";
-#else
-	strstream str(cmd.cmd, sizeof(cmd.cmd));
 #endif
-	char *qc;
-
-	cmd.port = id;
-	cmd.mode = TGI_EXEC_NORMAL;
-	cmd.cmd[0] = 0;
-	data.sleep.wakeup = timeout * 1000;
-	data.sleep.rings = 0;
-	data.sleep.loops = 1;
-
-	str << argv[0] << " query=";
-	strcpy(cmd.cmd, argv[0]);
-	strcat(cmd.cmd, " query=");
-	while(argv[argc])
-	{
-		if(argc > 1)
-			str << token;
-		str << argv[argc++];
-	}
-
-	str << " digits=" << dtmf.bin.data;
-
-	qc = getSymbol(SYM_CALLER);
-	if(qc)
-		str << " clid=" << qc;
-
-	qc = getSymbol(SYM_DIALED);
-	if(qc)
-		str << " dnid=" << qc;
-
-#ifdef	HAVE_SSTREAM
-	snprintf(cmd.cmd, sizeof(cmd.cmd), "%s", str.str().c_str());
-#else
-	str << ends;
-#endif
-	if(!timeout)
-		cmd.mode = TGI_EXEC_DETACH;
-	::write(tgipipe[1], &cmd, sizeof(cmd));
-
-	if(timeout)
-		trunkStep(TRUNK_STEP_SLEEP);
-	else
-		advance();
-
-}
 
 bool Trunk::isReady(void)
 {
@@ -366,10 +338,20 @@ void Trunk::setList(char **list)
 		{
 			if(!strnicmp(key, "server.", 7))
 				key = NULL;
-			else if(!strnicmp(key, "pstn.", 5))
+			else if(!stricmp(key, SYM_LOGIN))
+				key = NULL;
+			else if(!strnicmp(key, "driver.", 7))
+				key = NULL;
+			else if(!strnicmp(key, "user.", 5))
+				key = NULL;  
+			else if(!strnicmp(key, "line.", 5))
+				key = NULL;
+			else if(!strnicmp(key, "sql.", 4))
 				key = NULL;
 			else if(!strnicmp(key, "global.", 7))
 				key = NULL;
+			else if(!strnicmp(key, "rpc.", 4))
+				key = NULL;
 		}
 		if(key && value) 
 			setConst(key, urlDecode(value));
@@ -379,7 +361,21 @@ void Trunk::setList(char **list)
 
 void Trunk::commit(Symbol *sym)
 {
-	if(sym == &dtmf.bin)
+	const char *gid;
+
+	if(!stricmp(sym->id, "rpc.status"))
+	{
+		gid = getSymbol(SYM_GID);
+		if(!gid)
+			return;
+		gid = strchr(gid, '-');
+		if(gid)
+			++gid;
+		else
+			return;
+		rpclog.setInfo(gid, sym->data);
+	}
+	else if(sym == &dtmf.bin)
 		digits = strlen(dtmf.bin.data);
 	else
 		ScriptSymbol::commit(sym);
@@ -399,6 +395,8 @@ Script::Symbol *Trunk::getEntry(const char *name, int size)
 	struct tm tbuf;
 	Module *mod = Module::symFirst;
 	Script::Symbol *sym;
+	char buf[128];
+	char *login;
 
 	if(*name == '%')	// bug in interp
 		++name;
@@ -406,7 +404,53 @@ Script::Symbol *Trunk::getEntry(const char *name, int size)
         if(!strnicmp(name, "global.", 7))
                 return globals.getEntry(name, size);
 	else if(!strnicmp(name, "server.", 7))
-		return globals.getEntry(name, size); 
+		return globals.getEntry(name, 0); 
+	else if(!strnicmp(name, "line.", 5))
+	{
+		snprintf(buf, sizeof(buf), "line.%d.%s", id, name + 5);
+		sym = globals.getEntry(buf, 0);
+		if(!sym)
+		{
+			snprintf(buf, sizeof(buf), "line.default.%s", name + 5);
+			sym = globals.getEntry(buf, 0);
+		}
+		return sym;
+	}		 
+	else if(!strnicmp(name, "user.", 5))
+	{
+		login = getSymbol(SYM_LOGIN);
+		if(!strnicmp(name, "user.password", 13))
+		{
+			if(!isUser(login))
+				return globals.getEntry("password.nobody");
+
+			snprintf(buf, sizeof(buf), "%s.password", login);
+			sym = globals.getEntry(buf, 0);
+			if(!sym)
+				return globals.getEntry("password.new");
+			if(!stricmp(sym->data, "new"))
+				return globals.getEntry("password.new");
+			if(sym->flags.initial)
+				return globals.getEntry("password.new");
+			if(!sym->data[0] || !stricmp(sym->data, "none"))
+				return globals.getEntry("password.none");
+			return globals.getEntry("password.set");
+		} 
+		if(!isUser(login))
+			return NULL;
+
+		if(!stricmp(login, "none"))
+			return NULL;
+
+		snprintf(buf, sizeof(buf), "%s.%s", login, name + 5);
+		sym = globals.getEntry(buf, size);
+		if(!sym || sym->flags.initial)
+		{
+			snprintf(buf, sizeof(buf), "default.%s", name + 5);
+			sym = globals.getEntry(buf, size);
+		}
+		return sym;
+	} 
 
 	while(mod)
 	{
@@ -464,12 +508,7 @@ Script::Symbol *Trunk::getEntry(const char *name, int size)
 
 bool Trunk::attach(const char *name)
 {
-#ifndef	SCRIPT_LOCAL_DEFINE
-	ScriptCommand *img = getCommand();
-#endif
-
 	char buf[65];
-	Symbol *sym;
 	class Module *mod;
 	time_t now;
 	struct tm *dt;
@@ -484,6 +523,8 @@ bool Trunk::attach(const char *name)
 	TrunkEvent event;
 	const char *args[2];
 	char *e;
+	const char *login;
+	Symbol *sym;
 	
 
 	// This concerns me.  Why are we returning false?
@@ -512,6 +553,7 @@ bool Trunk::attach(const char *name)
 	flags.reset = flags.dtmf = flags.offhook = flags.once = flags.onexit = false;
 	tgi.pid = 0;
 	flags.digits = DTMF_MODE_LINE;
+	autoloop(true);
 
 	pid = getSymbol(SYM_PARENT);
 	if(pid)
@@ -527,7 +569,10 @@ bool Trunk::attach(const char *name)
 			mod = getModule(MODULE_XML, xml);
  
 		if(mod)
+		{
 			altimage = mod->getXML();
+			altmodule = mod;
+		}
 	}
 
 	if(strchr(name, '/') && altimage) {
@@ -551,11 +596,10 @@ bool Trunk::attach(const char *name)
 		return false;
 	}
 
-#ifdef	SCRIPT_LOCAL_DEFINE
 	ScriptImage *img = getImage();
-#endif
 
 	flags.script = true;
+
 	setSymbol(SYM_TONE, 16);
 	setSymbol(SYM_ANNOTATION, 160);
 	setSymbol(SYM_PLAYED, 12);
@@ -607,12 +651,12 @@ bool Trunk::attach(const char *name)
 	setSymbol(SYM_APPL, img->getLast("application"));
 	repSymbol(SYM_APPL, group->getLast("application"));
 
-	setSymbol(SYM_BUFFER, 8);
-	setSymbol(SYM_BUFFER, "8000");
-
 	setSymbol(SYM_VOLUME, 3);
 	setSymbol(SYM_VOLUME, group->getLast("volume"));
 
+	setSymbol(SYM_BUFFER, 8);
+	setSymbol(SYM_BUFFER, "8000");
+
 	setConst(SYM_NAME, "UNKNOWN");
 	setConst(SYM_CALLER, "UNKNOWN");
 	setConst(SYM_DIALED,  group->getNumber());
@@ -620,10 +664,35 @@ bool Trunk::attach(const char *name)
 	setConst(SYM_CLID, "UNKNOWN");
 	setConst(SYM_DNID, "UNKNOWN");
 
+	setSymbol(SYM_LOGIN, 16);
+
+	if(extNumber[0])
+	{
+		setConst(SYM_EXTNUMBER, extNumber);
+		setSymbol(SYM_LOGIN, extNumber);
+	}
+	else
+	{
+		login = keyserver.getLogin();
+		if(!stricmp(login, "none") || !stricmp(login, "admin"))
+			setSymbol(SYM_LOGIN, login);
+		else if(!stricmp(login, "port"))
+		{
+			snprintf(buf, 4, "%03d", id);
+			setSymbol(SYM_LOGIN, buf);
+		}
+		else
+			setSymbol(SYM_LOGIN, "none");
+	}
+
+	sym = getEntry(SYM_LOGIN, 0);
+	sym->flags.readonly = true;
+
 	initSyms();
 
 	sprintf(buf, "%03d", id);
 	setConst(SYM_ID, buf);
+	setConst(SYM_TRUNKID, buf);
 
 //	sprintf(buf, "%d", driver->getTrunkCount());
 //	setConst(SYM_PORTS, buf);
@@ -632,6 +701,21 @@ bool Trunk::attach(const char *name)
 
 	sprintf(buf, "%s-%03d-%010ld", keyserver.getNode(), id, now);
 	setConst(SYM_GID, buf);
+	setConst(SYM_CALLFWD, "none");
+	setSymbol(SYM_JOINID, 16);
+	setSymbol(SYM_PICKUP, 16);
+	setSymbol(SYM_RECALL, 16);
+	setSymbol(SYM_TRANSFER, 16);
+	setSymbol(SYM_TRUNK, 16);
+	setSymbol(SYM_INTERCOM, 16);
+	switch(flags.trunk)
+	{
+	case TRUNK_MODE_INCOMING:
+		setConst(SYM_CALLTYPE, "incoming");
+		break;
+	case TRUNK_MODE_OUTGOING:
+		setConst(SYM_CALLTYPE, "outgoing");
+	}
 
 	if(parent)
 	{
@@ -703,12 +787,17 @@ void Trunk::detach(void)
 	char buffer[256];
 	char overflow[256];
 	int i = 0;
-	char *val, *tag;
+	char *val, *tag, *gid;
 	unsigned len = 0;
 
 	if(!flags.script || !running)
 		return;
 
+	if(tonetmp)
+		delete tonetmp;
+
+	tonetmp = NULL;
+
 	++seq;
 	getName(buffer);
 //	slog(Slog::levelDebug) << buffer << ": hi water mark at detach; " << getPages() << endl;
@@ -756,6 +845,13 @@ void Trunk::detach(void)
 		child->postEvent(&event);
 	}
 
+	gid = getSymbol(SYM_GID);
+	if(gid)
+		gid = strchr(gid, '-');
+	
+	if(gid)
+		rpclog.stop(++gid);
+
 	dtmf.bin.data[0] = 0;
 	digits = 0;
 	cdrc = 0;
@@ -796,43 +892,53 @@ const char *Trunk::getPrefixPath(void)
 {
 	const char *prefix = getMember();
 
-	if(!prefix)
-		prefix = "";
+        if(!prefix)
+                prefix = "";
 
-	if(!stricmp(prefix, "feed"))
-		prefix = "memory";
-	else
-		prefix = getKeyword("prefix");
+        if(!stricmp(prefix, "feed"))
+                prefix = "memory";
+        else
+                prefix = getKeyword("prefix");
 
-	if(!prefix)
-		return prefix;
+        if(!prefix)
+                return prefix;
 
-	if(!stricmp(prefix, "memory"))
-		return keypaths.getLast("tmpfs");
-	return prefix;
+        if(!stricmp(prefix, "memory"))
+                return keypaths.getLast("tmpfs");
+        return prefix;
+}
+
+const char *Trunk::getExtReference(const char *ref)
+{
+	const char *cp = strchr(ref, '-');
+	Trunk *trunk;
+
+	if(!cp)
+		return ref;
+
+	trunk = driver->getTrunkPort(atoi(++cp));
+	if(!trunk)
+		return NULL;
+
+	if(!trunk->extNumber[0])
+		return NULL;
+
+	return trunk->extNumber;
 }
 
 timeout_t Trunk::getTimeout(const char *keywd)
 {
-#ifdef	SCRIPT_LOCAL_DEFINE
 	ScriptImage *img = getImage();
-#else
-	ScriptCommand *img = getCommand();
-#endif
 	if(keywd)
 		keywd = getKeyword(keywd);
 	if(!keywd)
-		keywd = getValue(img->getLast("timeout"));
+		keywd = getValue("86400");
 	return getSecTimeout(keywd);
 }
 
 timeout_t Trunk::getInterdigit(const char *keywd)
 {
-#ifdef	SCRIPT_LOCAL_DEFINE
 	ScriptImage *img = getImage();
-#else
-	ScriptCommand *img = getCommand();
-#endif
 
 	if(keywd)
 		keywd = getKeyword(keywd);
@@ -889,7 +995,7 @@ bool Trunk::trunkSignal(trunksignal_t signal)
 	{
 		line = getScript();
 		if(line->argc)
-			scrGoto();		
+                        scrGoto();
 		else
 			advance();
 		return true;
@@ -897,11 +1003,16 @@ bool Trunk::trunkSignal(trunksignal_t signal)
 
 	if(ScriptInterp::signal((unsigned long)(signal) - 1))
 	{
+		line = getScript();
 		if(signal == TRUNK_SIGNAL_HANGUP)
 		{
-			if(NULL != getScript())
+			autoloop(false);
+			if(line)
 				flags.onexit = true;
 		}
+		if(line)
+			if(line->method == (Method)&Trunk::scrRedirect)
+				(this->*(line->method))();
 		return true;
 	}
 
@@ -1026,6 +1137,313 @@ Script::Name *Trunk::getScriptImage(const char *name)
 }
 #endif
 
+#ifdef	SCRIPT_IF_OVERRIDE
+
+bool Trunk::ifFeature(ScriptInterp *interp, const char *v)
+{
+	if(!stricmp(v, "join") && (driver->getCaps() & Driver::capJoin))
+		return true;
+
+	if(!stricmp(v, "switch") && (driver->getCaps() & Driver::capSwitch))
+		return true;
+
+	if(!stricmp(v, "spans") && (driver->getCaps() & Driver::capSpans))
+		return true;
+
+	if(!stricmp(v, "speed") && (driver->getCaps() & Driver::capSpeed))
+		return true;
+
+	if(!stricmp(v, "gain") && (driver->getCaps() & Driver::capGain))
+		return true;
+
+	if(!stricmp(v, "pitch") && (driver->getCaps() & Driver::capPitch))
+		return true;
+
+	if(!stricmp(v, "tts") && hasTTS())
+		return true;
+
+#ifdef	HAVE_TGI
+	if(!stricmp(v, "tgi"))
+		return true;
+#else
+	if(!stricmp(v, "tgi"))
+		return false;
+#endif
+
+	if(!stricmp(v, "say") && tts)
+		return true;
+
+	if(!strnicmp(v, "conf", 4) && (driver->getCaps() & Driver::capConference))
+		return true;
+
+	if(!stricmp(v, "voice") && (((Trunk *)(interp))->getCapabilities() & TRUNK_CAP_VOICE))
+		return true;
+
+        if(!stricmp(v, "dial") && (((Trunk *)(interp))->getCapabilities() & TRUNK_CAP_DIAL))
+                return true;
+
+        if(!stricmp(v, "fax") && (((Trunk *)(interp))->getCapabilities() & (TRUNK_CAP_SENDFAX|TRUNK_CAP_RECVFAX)))
+                return true;
+
+        if(!stricmp(v, "data") && (((Trunk *)(interp))->getCapabilities() & TRUNK_CAP_DATA))
+                return true;
+
+        if(!stricmp(v, "station") && (((Trunk *)(interp))->getCapabilities() & TRUNK_CAP_STATION))
+                return true;
+
+	return false;
+}
+
+bool Trunk::ifDTMF(ScriptInterp *interp, const char *v)
+{
+	if(!stricmp(v, "line") && ((Trunk *)(interp))->flags.digits == DTMF_MODE_LINE)
+		return true;
+
+	if(!stricmp(v, "script") && ((Trunk *)(interp))->flags.digits == DTMF_MODE_SCRIPT)
+		return true;
+
+	if(!stricmp(v, "on") && ((Trunk *)(interp))->flags.digits == DTMF_MODE_ON)
+		return true;
+
+	if(!stricmp(v, "off") && ((Trunk *)(interp))->flags.digits == DTMF_MODE_OFF)
+		return true;
+
+	return false;
+}
+
+bool Trunk::isActiveUser(ScriptInterp *interp, const char *v)
+{
+	char namebuf[65];
+	Symbol *sym;
+
+	snprintf(namebuf, sizeof(namebuf), "%s.password", v);
+	sym = globals.getEntry(namebuf, 0);
+
+	if(!sym)
+		return false;
+
+	if(!sym->flags.initial)
+		return false;
+
+	return true;
+}
+
+bool Trunk::isHunt(ScriptInterp *interp, const char *v)
+{
+	char namebuf[65];
+
+	snprintf(namebuf, sizeof(namebuf), "hunt.%s.id", v);
+	if(globals.getSymbol(namebuf))
+		return true;
+
+	return false;
+}
+
+bool Trunk::isVirtual(ScriptInterp *interp, const char *v)
+{
+	char namebuf[33];
+	Symbol *sym;
+
+	v = getExtReference(v);
+
+	if(!v)
+		return false;
+
+	if(!isExtension(interp, v))
+		return false;
+
+	snprintf(namebuf, sizeof(namebuf), "%s.type", v);
+	sym = globals.getEntry(namebuf, 0);
+	if(!sym)
+		return false;
+
+	if(!stricmp(sym->data, "virtual"))
+		return true;
+
+	return false;
+}
+
+bool Trunk::isStation(ScriptInterp *interp, const char *v)
+{
+        char namebuf[33];
+        Symbol *sym;
+
+        v = getExtReference(v);
+
+        if(!v)
+                return false;
+
+        if(!isExtension(interp, v))
+                return false;
+
+        snprintf(namebuf, sizeof(namebuf), "%s.type", v);
+        sym = globals.getEntry(namebuf, 0);
+        if(!sym)
+                return false;
+
+        if(!stricmp(sym->data, "station"))
+                return true;
+
+        return false;
+}
+
+
+bool Trunk::isExtension(ScriptInterp *interp, const char *v)
+{
+	Trunk *trunk;
+
+	v = getExtReference(v);
+	
+	if(!v)
+		return false;
+
+	trunk = driver->getExtNumber(v);
+	if(!trunk)
+		return false;
+
+	return true;
+}	
+
+bool Trunk::isDnd(ScriptInterp *interp, const char *v)
+{
+        Trunk *trunk;
+
+        v = getExtReference(v);
+
+        if(!v)
+                return false;
+
+        trunk = driver->getExtNumber(v);
+        if(!trunk)
+                return false;
+
+        return trunk->flags.dnd;
+}
+
+bool Trunk::isService(ScriptInterp *interp, const char *v)
+{
+	const char *svc;
+
+	if(!stricmp(v, "up") && !service[0])
+		return true;
+
+	if(!service[0])
+		return false;
+
+        if(!stricmp(v, "test"))
+        {
+                if(!strnicmp(service, "test::", 6))
+                        return true;
+        }
+
+        if(!strnicmp(service, "test::", 6))
+                return false;
+
+
+	if(!stricmp(v, "down"))
+		return true;
+
+      	if(strchr(service, ':'))
+                svc += 2;
+        else
+                svc = service;
+
+        if(!stricmp(v, svc))
+                return true;
+
+        return false;
+}
+
+bool Trunk::isNode(ScriptInterp *interp, const char *v)
+{
+	const char *node = keyserver.getNode();
+	if(!node)
+		return false;
+
+	if(!stricmp(node, v))
+		return true;
+
+	return false;
+}
+
+bool Trunk::hasGroup(ScriptInterp *interp, const char *v)
+{
+	if(getGroup(v))
+		return true;
+
+	return false;
+}
+
+bool Trunk::hasPlugin(ScriptInterp *interp, const char *v)
+{
+	if(getModule(MODULE_ANY, v))
+		return true;
+
+	return false;
+}
+
+bool Trunk::hasVoice(ScriptInterp *interp, const char *v)
+{
+	const char *prompts = keypaths.getLast("prompts");
+	char buf[256];
+
+	snprintf(buf, sizeof(buf), "%s/%s", prompts, v);
+	if(isDir(buf))
+		return true;
+
+	return false;
+}
+
+bool Trunk::hasSysVoice(ScriptInterp *interp, const char *v)
+{
+        const char *prompts = keypaths.getLast("prompts");
+        char buf[256];
+
+        snprintf(buf, sizeof(buf), "%s/sys/%s", prompts, v);
+        if(isDir(buf))
+                return true;
+
+        return false;
+}
+
+bool Trunk::hasAltVoice(ScriptInterp *interp, const char *v)
+{
+        const char *prompts = keypaths.getLast("altprompts");
+        char buf[256];
+
+	if(!prompts)
+		prompts = keyserver.getPrefix();
+
+        snprintf(buf, sizeof(buf), "%s/%s", prompts, v);
+        if(isDir(buf))
+                return true;
+
+        return false;
+}
+
+bool Trunk::hasAppVoice(ScriptInterp *interp, const char *v)
+{
+        const char *prompts = keypaths.getLast("prompts");
+	Name *scr = interp->getObject();
+	char *cp;
+        char buf[256];
+	char name[65];
+
+	snprintf(name, sizeof(name), "%s", scr->name);
+	cp = strchr(name, ':');
+	if(cp)
+		*cp = 0;
+
+        snprintf(buf, sizeof(buf), "%s/%s/%s", prompts, v, name);
+        if(isDir(buf))
+                return true;
+
+        return false;
+}
+
+
+#endif
+
 #ifdef	CCXX_NAMESPACES
 };
 #endif
